from __future__ import annotations

import atexit
import gc
import logging
import os
from typing import TYPE_CHECKING, Any, Literal

import numpy as np
import torch

from mteb._requires_package import requires_package
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder
from mteb.types import PromptType

if TYPE_CHECKING:
    from collections.abc import Callable

    from torch.utils.data import DataLoader
    from vllm.config import PoolerConfig  # type: ignore[import-not-found]

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput


logger = logging.getLogger(__name__)

Dtype = Literal["half", "float16", "float", "float32", "bfloat16", "auto"]


class VllmWrapperBase:
    """Wrapper for vllm serving engine."""

    convert = "auto"
    mteb_model_meta: ModelMeta | None = None

    def __init__(
        self,
        model: str | ModelMeta,
        revision: str | None = None,
        *,
        trust_remote_code: bool = True,
        dtype: Dtype = "auto",
        head_dtype: Literal["model"] | Dtype | None = None,
        max_model_len: int | None = None,
        max_num_batched_tokens: int | None = None,
        max_num_seqs: int = 128,
        tensor_parallel_size: int = 1,
        enable_prefix_caching: bool | None = None,
        gpu_memory_utilization: float = 0.9,
        hf_overrides: dict[str, Any] | None = None,
        pooler_config: PoolerConfig | None = None,
        enforce_eager: bool = False,
        **kwargs: Any,
    ):
        """Wrapper for vllm serving engine.

        Args:
            model: model name string.
            revision: The revision of the model to use.
            trust_remote_code: Whether to trust remote code execution when loading the model.
                Should be True for models with custom code.
            dtype: Data type for model weights. "auto" will automatically select appropriate
                dtype based on hardware and model capabilities. vllm uses flash attention by
                default, which does not support fp32. Therefore, it defaults to using fp16 for
                inference on fp32 models. Testing has shown a relatively small drop in accuracy.
                You can manually opt for fp32, but inference speed will be very slow.
            head_dtype: "head" refers to the last Linear layer(s) of an LLMs, such as the score
                or classifier in a classification model. Uses fp32 for the head by default to
                gain extra precision.
            max_model_len: Maximum sequence length (context window) supported by the model.
                If None, uses the model's default maximum length.
            max_num_batched_tokens: Maximum number of tokens to process in a single batch.
                If None, automatically determined.
            max_num_seqs: Maximum number of sequences to process concurrently.
            tensor_parallel_size: Number of GPUs for tensor parallelism.
            enable_prefix_caching: Whether to enable KV cache sharing for common prompt prefixes.
                If None, uses the model's default setting.
            gpu_memory_utilization: Target GPU memory utilization ratio (0.0 to 1.0).
            hf_overrides: Dictionary mapping Hugging Face configuration keys to override values.
            pooler_config: Controls the behavior of output pooling in pooling models.
            enforce_eager: Whether to disable CUDA graph optimization and use eager execution.
            **kwargs: Additional arguments to pass to the vllm serving engine model.
        """
        requires_package(
            self,
            "vllm",
            "Wrapper for vllm serving engine",
            install_instruction="pip install mteb[vllm]",
        )

        os.environ["VLLM_WORKER_MULTIPROC_METHOD"] = "spawn"

        from vllm import LLM, EngineArgs

        hf_overrides = {} if hf_overrides is None else hf_overrides

        if head_dtype is not None:
            hf_overrides["head_dtype"] = head_dtype

        model_name = model if isinstance(model, str) else model.name

        if isinstance(model, ModelMeta):
            logger.info(
                "Using revision from model meta. Passed revision will be ignored"
            )
            revision = model.revision

        args = EngineArgs(
            model=model_name,
            revision=revision,
            runner="pooling",
            convert=self.convert,  # type: ignore[arg-type]
            max_model_len=max_model_len,
            max_num_batched_tokens=max_num_batched_tokens,
            max_num_seqs=max_num_seqs,
            tensor_parallel_size=tensor_parallel_size,
            enable_prefix_caching=enable_prefix_caching,
            gpu_memory_utilization=gpu_memory_utilization,
            hf_overrides=hf_overrides,
            pooler_config=pooler_config,
            enforce_eager=enforce_eager,
            trust_remote_code=trust_remote_code,
            dtype=dtype,
            **kwargs,
        )
        self.llm = LLM(**vars(args))

        if isinstance(model, str):
            self.mteb_model_meta = ModelMeta.from_hub(model=model, revision=revision)
        else:
            self.mteb_model_meta = model

        atexit.register(self.cleanup)

    def cleanup(self):
        """Clean up the VLLM distributed runtime environment and release GPU resources."""
        if self.llm is None:
            return

        from vllm.distributed import (  # type: ignore[import-not-found]
            cleanup_dist_env_and_memory,
        )

        self.llm = None
        gc.collect()
        cleanup_dist_env_and_memory()

    def __del__(self):
        try:
            self.cleanup()
        except Exception:
            pass


class VllmEncoderWrapper(AbsEncoder, VllmWrapperBase):
    """vLLM wrapper for Encoder models.

    Args:
        model: model name string or ModelMeta.
        revision: The revision of the model to use.
        prompt_dict: A dictionary mapping task names to prompt strings.
        use_instructions: Whether to use instructions from the prompt_dict.
            When False, values from prompt_dict are used as static prompts (prefixes).
            When True, values from prompt_dict are used as instructions to be formatted
            using the instruction_template.
        instruction_template: A template or callable to format instructions.
            Can be a string with '{instruction}' placeholder or a callable that takes
            the instruction and prompt type and returns a formatted string.
        apply_instruction_to_documents: Whether to apply instructions to documents prompts.
        **kwargs: Additional arguments to pass to the vllm serving engine model.
    """

    convert = "embed"

    def __init__(
        self,
        model: str | ModelMeta,
        revision: str | None = None,
        prompt_dict: dict[str, str] | None = None,
        use_instructions: bool = False,
        instruction_template: (
            str | Callable[[str, PromptType | None], str] | None
        ) = None,
        apply_instruction_to_documents: bool = True,
        **kwargs: Any,
    ):
        if use_instructions and instruction_template is None:
            raise ValueError(
                "To use instructions, an instruction_template must be provided. "
                "For example, `Instruction: {instruction}`"
            )

        if (
            isinstance(instruction_template, str)
            and "{instruction}" not in instruction_template
        ):
            raise ValueError(
                "Instruction template must contain the string '{instruction}'."
            )

        self.prompts_dict = prompt_dict
        self.use_instructions = use_instructions
        self.instruction_template = instruction_template
        self.apply_instruction_to_passages = apply_instruction_to_documents
        super().__init__(
            model,
            revision,
            **kwargs,
        )

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        """Encodes the given sentences using the encoder.

        Args:
            inputs: The sentences to encode.
            task_metadata: The metadata of the task. Sentence-transformers uses this to
                determine which prompt to use from a specified dictionary.
            prompt_type: The name type of prompt. (query or passage)
            hf_split: Split of current task
            hf_subset: Subset of current task
            **kwargs: Additional arguments to pass to the encoder.

        Returns:
            The encoded sentences.
        """
        prompt = ""
        if self.use_instructions and self.prompts_dict is not None:
            prompt = self.get_task_instruction(task_metadata, prompt_type)
        elif self.prompts_dict is not None:
            prompt_name = self.get_prompt_name(task_metadata, prompt_type)
            if prompt_name is not None:
                prompt = self.prompts_dict.get(prompt_name, "")

        if (
            self.use_instructions
            and self.apply_instruction_to_passages is False
            and prompt_type == PromptType.document
        ):
            logger.info(
                f"No instruction used, because prompt type = {prompt_type.document}"
            )
            prompt = ""
        else:
            logger.info(
                f"Using instruction: '{prompt}' for task: '{task_metadata.name}' prompt type: '{prompt_type}'"
            )

        prompts = [prompt + text for batch in inputs for text in batch["text"]]
        outputs = self.llm.encode(
            prompts, pooling_task="embed", truncate_prompt_tokens=-1
        )
        embeddings = torch.stack([output.outputs.data for output in outputs])
        return embeddings


class VllmCrossEncoderWrapper(VllmWrapperBase):
    """vLLM wrapper for CrossEncoder models."""

    convert = "classify"

    def __init__(
        self,
        model: str | ModelMeta,
        revision: str | None = None,
        query_prefix: str = "",
        document_prefix: str = "",
        **kwargs: Any,
    ):
        super().__init__(
            model,
            revision,
            **kwargs,
        )
        self.query_prefix = query_prefix
        self.document_prefix = document_prefix

    def predict(
        self,
        inputs1: DataLoader[BatchedInput],
        inputs2: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        """Predicts relevance scores for pairs of inputs. Note that, unlike the encoder, the cross-encoder can compare across inputs.

        Args:
            inputs1: First Dataloader of inputs to encode. For reranking tasks, these are queries (for text only tasks `QueryDatasetType`).
            inputs2: Second Dataloader of inputs to encode. For reranking, these are documents (for text only tasks `RetrievalOutputType`).
            task_metadata: Metadata of the current task.
            hf_split: Split of current task, allows to know some additional information about current split.
                E.g. Current language
            hf_subset: Subset of current task. Similar to `hf_split` to get more information
            prompt_type: The name type of prompt. (query or passage)
            **kwargs: Additional arguments to pass to the cross-encoder.

        Returns:
            The predicted relevance scores for each inputs pair.
        """
        queries = [
            self.query_prefix + text for batch in inputs1 for text in batch["text"]
        ]
        corpus = [
            self.document_prefix + text for batch in inputs2 for text in batch["text"]
        ]
        # TODO: support score prompt

        outputs = self.llm.score(
            queries,
            corpus,
            truncate_prompt_tokens=-1,
            use_tqdm=False,
        )
        scores = np.array([output.outputs.score for output in outputs])
        return scores



from mteb.abstasks.multilabel_classification import (
    AbsTaskMultilabelClassification,
)
from mteb.abstasks.task_metadata import TaskMetadata


class BrazilianToxicTweetsClassification(AbsTaskMultilabelClassification):
    metadata = TaskMetadata(
        name="BrazilianToxicTweetsClassification",
        description="ToLD-Br is the biggest dataset for toxic tweets in Brazilian Portuguese, crowdsourced by 42 annotators selected from a pool of 129 volunteers. Annotators were selected aiming to create a plural group in terms of demographics (ethnicity, sexual orientation, age, gender). Each tweet was labeled by three annotators in 6 possible categories: LGBTQ+phobia, Xenophobia, Obscene, Insult, Misogyny and Racism.",
        reference="https://paperswithcode.com/dataset/told-br",
        dataset={
            "path": "mteb/BrazilianToxicTweetsClassification",
            "revision": "d7af1f3c9ff00fbb4e1e40021788a32d78a936bd",
        },
        type="MultilabelClassification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["por-Latn"],
        main_score="accuracy",
        date=("2019-08-01", "2019-08-16"),
        domains=["Constructed", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=["brazilian"],
        sample_creation="found",
        bibtex_citation=r"""
@article{DBLP:journals/corr/abs-2010-04543,
  author = {Joao Augusto Leite and
Diego F. Silva and
Kalina Bontcheva and
Carolina Scarton},
  eprint = {2010.04543},
  eprinttype = {arXiv},
  journal = {CoRR},
  timestamp = {Tue, 15 Dec 2020 16:10:16 +0100},
  title = {Toxic Language Detection in Social Media for Brazilian Portuguese:
New Dataset and Multilingual Analysis},
  url = {https://arxiv.org/abs/2010.04543},
  volume = {abs/2010.04543},
  year = {2020},
}
""",
    )



from .turkish_constitutional_court import (
    TurkishConstitutionalCourtViolation,
)
from .turkish_movie_sentiment_classification import (
    TurkishMovieSentimentClassification,
    TurkishMovieSentimentClassificationV2,
)
from .turkish_product_sentiment_classification import (
    TurkishProductSentimentClassification,
    TurkishProductSentimentClassificationV2,
)

__all__ = [
    "TurkishConstitutionalCourtViolation",
    "TurkishMovieSentimentClassification",
    "TurkishMovieSentimentClassificationV2",
    "TurkishProductSentimentClassification",
    "TurkishProductSentimentClassificationV2",
]



from mteb.models.model_meta import ModelMeta
from mteb.models.sentence_transformer_wrapper import SentenceTransformerEncoderWrapper

codemodernbert_crow_meta = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="Shuu12121/CodeSearch-ModernBERT-Crow-Plus",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="044a7a4b552f86e284817234c336bccf16f895ce",
    release_date="2025-04-21",
    n_parameters=151668480,
    n_embedding_parameters=38403072,
    memory_usage_mb=607,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=1024,
    reference="https://huggingface.co/Shuu12121/CodeSearch-ModernBERT-Crow-Plus",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        "CodeSearchNetRetrieval",
        # "code-search-net/code_search_net",
        # "Shuu12121/python-codesearch-filtered",
        # "Shuu12121/java-codesearch-filtered",
        # "Shuu12121/javascript-codesearch-filtered",
        # "Shuu12121/ruby-codesearch-filtered",
        # "Shuu12121/rust-codesearch-filtered",
    },
)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import numpy as np
import torch
from tqdm.auto import tqdm

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_package
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput, TextInput


class MuQMuLanWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str = "OpenMuQ/MuQ-MuLan-large",
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_s: float = 30.0,
        **kwargs: Any,
    ):
        requires_package(self, "muq", "pip install 'mteb[muq]'")
        from muq import MuQMuLan

        self.model_name = model_name
        self.device = device
        self.sampling_rate = 24000
        self.max_audio_length_s = max_audio_length_s
        # Apply audio truncation (30 seconds max)
        self.max_length_samples = int(self.max_audio_length_s * self.sampling_rate)

        # Load the model
        self.model = MuQMuLan.from_pretrained(model_name).eval().to(self.device)
        self.model.eval()

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        inputs.collate_fn = AudioCollator(self.sampling_rate)

        all_features = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            audio_arrays = []
            audio_array = [audio["array"] for audio in batch["audio"]]
            for array in audio_array:
                # Apply audio truncation (30 seconds max)
                if array.shape[-1] > self.max_length_samples:
                    array = array[..., : self.max_length_samples]
                audio_arrays.append(array)

            # Find max length and pad all tensors
            max_length = max(arr.shape[-1] for arr in audio_arrays)
            batch_tensor = torch.zeros(
                len(audio_arrays), max_length, dtype=torch.float32
            )

            for idx, arr in enumerate(audio_arrays):
                length = arr.shape[-1]
                batch_tensor[idx, :length] = arr

            batch_tensor = batch_tensor.to(self.device)

            with torch.no_grad():
                # Process entire batch at once
                audio_embeds = self.model(wavs=batch_tensor)
                all_features.extend(
                    [
                        embed.cpu().detach().numpy().reshape(1, -1)
                        for embed in audio_embeds
                    ]
                )

        return np.vstack(all_features)

    def get_text_embeddings(
        self,
        inputs: DataLoader[TextInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> np.ndarray:
        """Get text embeddings using MuQ-MuLan."""
        all_embeddings = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
            desc="Processing text batches",
        ):
            texts = batch["text"]
            with torch.no_grad():
                text_embeds = self.model(texts=texts)
                all_embeddings.append(text_embeds.cpu().detach().numpy())

        return np.vstack(all_embeddings)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        audio_embeddings = None

        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "audio" in inputs.dataset.features:
            audio_embeddings = self.get_audio_embeddings(inputs, **kwargs)

        if text_embeddings is not None and audio_embeddings is not None:
            if len(text_embeddings) != len(audio_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + audio_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif audio_embeddings is not None:
            return audio_embeddings
        raise ValueError

    def similarity(
        self,
        embeddings1: Array,
        embeddings2: Array,
    ) -> Array:
        """Calculate similarity between audio and text embeddings."""
        embeddings1 = torch.from_numpy(embeddings1).to(self.device)
        embeddings2 = torch.from_numpy(embeddings2).to(self.device)

        with torch.no_grad():
            similarity = self.model.calc_similarity(embeddings1, embeddings2)

        return similarity.cpu().detach().numpy()


muq_mulan_large = ModelMeta(
    loader=MuQMuLanWrapper,
    name="OpenMuQ/MuQ-MuLan-large",
    languages=["eng-Latn", "zho-Hans"],  # English and Chinese support
    revision="8a081dbcf84edd47ea7db3c4ecb8fd1ec1ddacfe",
    release_date="2025-01-01",
    modalities=["audio", "text"],
    n_parameters=630_000_000,
    memory_usage_mb=2530,
    max_tokens=None,
    embed_dim=512,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="https://github.com/tencent-ailab/MuQ",
    public_training_data="https://github.com/tencent-ailab/MuQ",
    framework=["PyTorch"],
    reference="https://huggingface.co/OpenMuQ/MuQ-MuLan-large",
    # https://github.com/tencent-ailab/MuQ/blob/28847ea50cd31ac4b8b6a7dacc051ad7d1c7606a/src/muq/muq_mulan/muq_mulan.py#L171
    similarity_fn_name="dot",
    use_instructions=False,
    training_datasets=set(),
    citation="""
@misc{zhu2025muqselfsupervisedmusicrepresentation,
  title={MuQ: Self-Supervised Music Representation Learning with Mel Residual Vector Quantization},
  author={Haina Zhu and Yizhi Zhou and Hangting Chen and Jianwei Yu and Ziyang Ma and Rongzhi Gu and Yi Luo and Wei Tan and Xie Chen},
  year={2025},
  eprint={2501.01108},
  archivePrefix={arXiv},
  primaryClass={cs.SD},
  url={https://arxiv.org/abs/2501.01108},
}""",
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class VisRAGRetArxivQA(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="VisRAGRetArxivQA",
        description="evaluate vision-based retrieval and generation on scientific figures and their surrounding context to preserve complex layouts and mathematical notations.",
        reference="https://arxiv.org/abs/2403.00231",
        type="Retrieval",
        task_subtypes=["Image Text Retrieval"],
        category="t2i",
        modalities=["text", "image"],
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="recall_at_10",
        dataset={
            "path": "mteb/VisRAGRetArxivQA",
            "revision": "b0da0f3f9677461eb78e34a7164596ff4f86bd52",
        },
        date=("2000-01-01", "2024-12-31"),
        domains=["Academic", "Non-fiction"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{li2024multimodalarxivdatasetimproving,
  archiveprefix = {arXiv},
  author = {Lei Li and Yuqi Wang and Runxin Xu and Peiyi Wang and Xiachong Feng and Lingpeng Kong and Qi Liu},
  eprint = {2403.00231},
  primaryclass = {cs.CV},
  title = {Multimodal ArXiv: A Dataset for Improving Scientific Comprehension of Large Vision-Language Models},
  url = {https://arxiv.org/abs/2403.00231},
  year = {2024},
}""",
    )



from .german_politicians_twitter_sentiment_classification import (
    GermanPoliticiansTwitterSentimentClassification,
    GermanPoliticiansTwitterSentimentClassificationV2,
)
from .ten_k_gnad_classification import TenKGnadClassification, TenKGnadClassificationV2

__all__ = [
    "GermanPoliticiansTwitterSentimentClassification",
    "GermanPoliticiansTwitterSentimentClassificationV2",
    "TenKGnadClassification",
    "TenKGnadClassificationV2",
]



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.zeroshot_classification import (
    AbsTaskZeroShotClassification,
)


class OxfordPetsZeroShotClassification(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="OxfordPetsZeroShot",
        description="Classifying animal images.",
        reference="https://arxiv.org/abs/1306.5151",
        dataset={
            "path": "mteb/OxfordPets",
            "revision": "557b480fae8d69247be74d9503b378a09425096f",
        },
        type="ZeroShotClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2009-01-01",
            "2010-01-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation=r"""
@misc{maji2013finegrainedvisualclassificationaircraft,
  archiveprefix = {arXiv},
  author = {Subhransu Maji and Esa Rahtu and Juho Kannala and Matthew Blaschko and Andrea Vedaldi},
  eprint = {1306.5151},
  primaryclass = {cs.CV},
  title = {Fine-Grained Visual Classification of Aircraft},
  url = {https://arxiv.org/abs/1306.5151},
  year = {2013},
}
""",
    )

    def get_candidate_labels(self) -> list[str]:
        return [
            f"a photo of a {name}, a type of pet."
            for name in self.dataset["test"].features[self.label_column_name].names
        ]



"""Implementation of Sentence Transformers model validated in MTEB."""

import numpy as np

from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import SentenceTransformerEncoderWrapper

paraphrase_langs = [
    "ara-Arab",
    "bul-Cyrl",
    "cat-Latn",
    "ces-Latn",
    "dan-Latn",
    "deu-Latn",
    "ell-Grek",
    "eng-Latn",
    "spa-Latn",
    "est-Latn",
    "fas-Arab",
    "fin-Latn",
    "fra-Latn",
    "fra-Latn",
    "glg-Latn",
    "guj-Gujr",
    "heb-Hebr",
    "hin-Deva",
    "hrv-Latn",
    "hun-Latn",
    "hye-Armn",
    "ind-Latn",
    "ita-Latn",
    "jpn-Jpan",
    "kat-Geor",
    "kor-Hang",
    "kur-Arab",
    "lit-Latn",
    "lav-Latn",
    "mkd-Cyrl",
    "mon-Cyrl",
    "mar-Deva",
    "msa-Latn",
    "mya-Mymr",
    "nob-Latn",
    "nld-Latn",
    "pol-Latn",
    "por-Latn",
    "por-Latn",
    "ron-Latn",
    "rus-Cyrl",
    "slk-Latn",
    "slv-Latn",
    "sqi-Latn",
    "srp-Cyrl",
    "swe-Latn",
    "tha-Thai",
    "tur-Latn",
    "ukr-Cyrl",
    "urd-Arab",
    "vie-Latn",
    "zho-Hans",
    "zho-Hant",
]

SBERT_CITATION = """@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "http://arxiv.org/abs/1908.10084",
}
"""


sent_trf_training_dataset = {
    # derived from datasheets
    "MSMARCO",
    "MSMARCOHardNegatives",
    "NanoMSMARCORetrieval",
    "MSMARCO-PL",  # translation not trained on
    "mMARCO-NL",  # translation not trained on
    "NQ",
    "NQHardNegatives",
    "NanoNQRetrieval",
    "NQ-PL",  # translation not trained on
    "NQ-NL",  # translation not trained on
    # not in MTEB
    # "s2orc",
    # "flax-sentence-embeddings/stackexchange_xml",
    # "ms_marco",
    # "gooaq",
    # "yahoo_answers_topics",
    # "code_search_net",
    # "search_qa",
    # "eli5",
    # "snli",
    # "multi_nli",
    # "wikihow",
    # "natural_questions",
    # "trivia_qa",
    # "embedding-data/sentence-compression",
    # "embedding-data/flickr30k-captions",
    # "embedding-data/altlex",
    # "embedding-data/simple-wiki",
    # "embedding-data/QQP",
    # "embedding-data/SPECTER",
    # "embedding-data/PAQ_pairs",
    # "embedding-data/WikiAnswers",
}

all_minilm_l6_v2 = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/all-MiniLM-L6-v2",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="8b3219a92973c328a8e22fadcfa821b5dc75636a",
    release_date="2021-08-30",
    n_parameters=22713216,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=87,
    embed_dim=384,
    license="apache-2.0",
    max_tokens=256,
    reference="https://huggingface.co/sentence-transformers/all-MiniLM-L6-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "ONNX",
        "safetensors",
        "Transformers",
    ],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    training_datasets=sent_trf_training_dataset,
    public_training_code=None,
    public_training_data=None,
    citation=SBERT_CITATION,
)

all_minilm_l12_v2 = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/all-MiniLM-L12-v2",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="364dd28d28dcd3359b537f3cf1f5348ba679da62",
    release_date="2021-08-30",
    n_parameters=33360000,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=127,
    embed_dim=384,
    license="apache-2.0",
    max_tokens=256,
    reference="https://huggingface.co/sentence-transformers/all-MiniLM-L12-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "ONNX",
        "safetensors",
        "Transformers",
    ],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    training_datasets=sent_trf_training_dataset,
    public_training_code=None,
    citation=SBERT_CITATION,
    public_training_data=None,
)

paraphrase_multilingual_minilm_l12_v2 = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2",
    model_type=["dense"],
    languages=paraphrase_langs,
    open_weights=True,
    revision="bf3bf13ab40c3157080a7ab344c831b9ad18b5eb",
    release_date="2019-11-01",  # release date of paper
    n_parameters=117653760,
    n_embedding_parameters=96_014_208,
    memory_usage_mb=449,
    embed_dim=384,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "ONNX",
        "safetensors",
        "Transformers",
    ],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    training_datasets=sent_trf_training_dataset,  # assumed (probably some parallel as well)
    public_training_code=None,
    citation=SBERT_CITATION,
    public_training_data=None,
)

paraphrase_multilingual_mpnet_base_v2 = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
    model_type=["dense"],
    languages=paraphrase_langs,
    open_weights=True,
    revision="79f2382ceacceacdf38563d7c5d16b9ff8d725d6",
    release_date="2019-11-01",  # release date of paper
    n_parameters=278043648,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=1061,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "ONNX",
        "safetensors",
        "Transformers",
    ],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    citation=SBERT_CITATION,
    training_datasets=sent_trf_training_dataset,
    # + https://github.com/UKPLab/sentence-transformers/blob/master/examples/training/paraphrases/training.py
    # which include (not in MTEB):
    # "all-nli": all_nli_train_dataset,
    # "sentence-compression": sentence_compression_train_dataset,
    # "simple-wiki": simple_wiki_train_dataset,
    # "altlex": altlex_train_dataset,
    # "quora-duplicates": quora_train_dataset,
    # "coco-captions": coco_train_dataset,
    # "flickr30k-captions": flickr_train_dataset,
    # "yahoo-answers": yahoo_answers_train_dataset,
    # "stack-exchange": stack_exchange_train_dataset,
    public_training_code=None,
    public_training_data=None,
)

labse = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/LaBSE",
    model_type=["dense"],
    languages=paraphrase_langs,
    open_weights=True,
    revision="e34fab64a3011d2176c99545a93d5cbddc9a91b7",
    release_date="2019-11-01",  # release date of paper
    n_parameters=470926848,
    n_embedding_parameters=384_885_504,
    memory_usage_mb=1796,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/LaBSE",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    training_datasets=set(
        # CommonCrawl
        # wiki  05-21-2020 dump
        # The translation corpus is constructed from web pages using a bitext mining system
    ),
    # scraped and mined webdata including CC, wiki, see section 3.1 https://aclanthology.org/2022.acl-long.62.pdf
    public_training_code="https://www.kaggle.com/models/google/labse/tensorFlow2/labse/2?tfhub-redirect=true",
    citation="""@misc{feng2022languageagnosticbertsentenceembedding,
      title={Language-agnostic BERT Sentence Embedding},
      author={Fangxiaoyu Feng and Yinfei Yang and Daniel Cer and Naveen Arivazhagan and Wei Wang},
      year={2022},
      eprint={2007.01852},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2007.01852},
    }""",
    public_training_data=None,
)

multi_qa_minilm_l6_cos_v1 = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/multi-qa-MiniLM-L6-cos-v1",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="b207367332321f8e44f96e224ef15bc607f4dbf0",
    release_date="2021-08-30",
    n_parameters=22713216,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=87,
    embed_dim=384,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/multi-qa-MiniLM-L6-cos-v1",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "ONNX",
        "safetensors",
        "Transformers",
    ],
    use_instructions=False,
    superseded_by=None,
    adapted_from="nreimers/MiniLM-L6-H384-uncased",
    training_datasets=sent_trf_training_dataset,  # assumed
    public_training_code=None,
    public_training_data=None,
    citation=SBERT_CITATION,
)

all_mpnet_base_v2 = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/all-mpnet-base-v2",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="9a3225965996d404b775526de6dbfe85d3368642",
    release_date="2021-08-30",
    n_parameters=109486464,
    n_embedding_parameters=23_444_736,
    memory_usage_mb=418,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=384,
    reference="https://huggingface.co/sentence-transformers/all-mpnet-base-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "ONNX",
        "safetensors",
        "Transformers",
    ],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    training_datasets=sent_trf_training_dataset,
    public_training_code=None,
    public_training_data=None,
    citation=SBERT_CITATION,
)

static_multi_datasets = {
    # wikititles
    "TatoebaBitextMining",
    # talks
    # europarl
    # global_voices
    # muse
    # wikimatrix
    # opensubtitles
    "StackExchangeClustering.v2",
    "StackExchangeClusteringP2P.v2",
    # quora
    "QuoraRetrieval",
    "NanoQuoraRetrieval",
    "Quora-NL",  # translation not trained on
    # wikianswers_duplicates
    # all_nli - the "triplet" subset
    # simple_wiki
    # altlex
    # flickr30k_captions
    # coco_captions
    # nli_for_simcse
    # negation
}
static_multi_languages = [
    "eng-Latn",
    "ara-Arab",
    "bul-Cyrl",
    "cat-Latn",
    "ces-Latn",
    "dan-Latn",
    "deu-Latn",
    "ell-Grek",
    "spa-Latn",
    "est-Latn",
    "fas-Arab",
    "fin-Latn",
    "fra-Latn",
    "glg-Latn",
    "guj-Gujr",
    "heb-Hebr",
    "hin-Deva",
    "hun-Latn",
    "hye-Armn",
    "ind-Latn",
    "ita-Latn",
    "jpn-Jpan",
    "kat-Geor",
    "kor-Hang",
    "kur-Latn",
    "lit-Latn",
    "lav-Latn",
    "mkd-Cyrl",
    "mon-Cyrl",
    "mar-Deva",
    "mal-Mlym",
    "mya-Mymr",
    "nob-Latn",
    "nld-Latn",
    "pol-Latn",
    "por-Latn",
    "ron-Latn",
    "rus-Cyrl",
    "slk-Latn",
    "slv-Latn",
    "sqi-Latn",
    "srp-Cyrl",
    "swe-Latn",
    "tha-Thai",
    "tur-Latn",
    "ukr-Cyrl",
    "urd-Arab",
    "vie-Latn",
    "zho-Hans",
]

static_similarity_mrl_multilingual_v1 = ModelMeta(
    name="sentence-transformers/static-similarity-mrl-multilingual-v1",
    model_type=["dense"],
    loader=SentenceTransformerEncoderWrapper,
    loader_kwargs=dict(
        device="cpu",  # CPU is just as quick, if not quicker
    ),
    languages=static_multi_languages,
    open_weights=True,
    revision="7264ea07c5365a11d7e6d87dbb6195889a13054f",
    release_date="2025-01-15",
    n_parameters=108_420_096,
    n_embedding_parameters=108420096,
    memory_usage_mb=413,
    embed_dim=1024,
    license="apache-2.0",
    max_tokens=None,
    reference="https://huggingface.co/sentence-transformers/static-similarity-mrl-multilingual-v1",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    training_datasets=static_multi_datasets,
    public_training_code="https://huggingface.co/blog/static-embeddings",
    public_training_data="https://huggingface.co/collections/sentence-transformers/embedding-model-datasets-6644d7a3673a511914aa7552",
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)

contriever = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="facebook/contriever-msmarco",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="abe8c1493371369031bcb1e02acb754cf4e162fa",
    release_date="2022-06-25",  # release date of model on HF
    n_parameters=109482240,
    n_embedding_parameters=23_440_896,
    memory_usage_mb=572,
    embed_dim=768,
    license=None,
    max_tokens=512,
    reference="https://huggingface.co/facebook/contriever-msmarco",
    similarity_fn_name=ScoringFunction.DOT_PRODUCT,
    framework=["Sentence Transformers", "PyTorch", "Transformers"],
    use_instructions=False,
    citation="""
    @misc{izacard2021contriever,
      title={Unsupervised Dense Information Retrieval with Contrastive Learning},
      author={Gautier Izacard and Mathilde Caron and Lucas Hosseini and Sebastian Riedel and Piotr Bojanowski and Armand Joulin and Edouard Grave},
      year={2021},
      url = {https://arxiv.org/abs/2112.09118},
      doi = {10.48550/ARXIV.2112.09118},
    }""",
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
)

microllama_text_embedding = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="keeeeenw/MicroLlama-text-embedding",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="98f70f14cdf12d7ea217ed2fd4e808b0195f1e7e",
    release_date="2024-11-10",
    n_parameters=271869952,
    n_embedding_parameters=32_769_024,
    memory_usage_mb=1037,
    embed_dim=1024,
    license="apache-2.0",
    max_tokens=2048,
    reference="https://huggingface.co/keeeeenw/MicroLlama-text-embedding",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    training_datasets={
        "NQ",
        "NQHardNegatives",
        "NanoNQRetrieval",
        "NQ-PL",  # translation not trained on
        "NQ-NL",  # translation not trained on
        # not in MTEB
        # "sentence-transformers/all-nli",
        # "sentence-transformers/stsb",
        # "sentence-transformers/quora-duplicates",
        # "sentence-transformers/natural-questions",
    },
    public_training_code=None,
    public_training_data=None,
)

SENTENCE_T5_CITATION = """
@misc{ni2021sentencet5scalablesentenceencoders,
      title={Sentence-T5: Scalable Sentence Encoders from Pre-trained Text-to-Text Models},
      author={Jianmo Ni and Gustavo Hernández Ábrego and Noah Constant and Ji Ma and Keith B. Hall and Daniel Cer and Yinfei Yang},
      year={2021},
      eprint={2108.08877},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2108.08877},
}
"""
sentence_t5_base = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/sentence-t5-base",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="50c53e206f8b01c9621484a3c0aafce4e55efebf",
    release_date="2022-02-09",
    n_parameters=222903552,
    n_embedding_parameters=24_674_304,
    memory_usage_mb=209,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/sentence-t5-base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={"SNLI", "Community QA"},
    citation=SENTENCE_T5_CITATION,
)

sentence_t5_large = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/sentence-t5-large",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="1fc08ea477205aa54a3e5b13f0971ae16b86410a",
    release_date="2022-02-09",
    n_parameters=335726080,
    n_embedding_parameters=32_899_072,
    memory_usage_mb=639,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/sentence-t5-large",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={"SNLI", "Community QA"},
    citation=SENTENCE_T5_CITATION,
)

sentence_t5_xl = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/sentence-t5-xl",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="2965d31b368fb14117688e0bde77cbd720e91f53",
    release_date="2024-03-27",
    n_parameters=1241696256,
    n_embedding_parameters=32_899_072,
    memory_usage_mb=2367,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/sentence-t5-xl",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={"SNLI", "Community QA"},
    citation=SENTENCE_T5_CITATION,
)

sentence_t5_xxl = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/sentence-t5-xxl",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="4d122282ba80e807e9e6eb8c358269e92796365d",
    release_date="2024-03-27",
    n_parameters=4865577984,
    n_embedding_parameters=32899072,
    memory_usage_mb=9279,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/sentence-t5-xxl",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={"SNLI", "Community QA"},
    citation=SENTENCE_T5_CITATION,
)
GTR_CITATION = """
@misc{ni2021largedualencodersgeneralizable,
      title={Large Dual Encoders Are Generalizable Retrievers},
      author={Jianmo Ni and Chen Qu and Jing Lu and Zhuyun Dai and Gustavo Hernández Ábrego and Ji Ma and Vincent Y. Zhao and Yi Luan and Keith B. Hall and Ming-Wei Chang and Yinfei Yang},
      year={2021},
      eprint={2112.07899},
      archivePrefix={arXiv},
      primaryClass={cs.IR},
      url={https://arxiv.org/abs/2112.07899},
}
"""
gtr_t5_large = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/gtr-t5-large",
    model_type=["dense"],
    languages=["eng-Latn"],  # in format eng-Latn
    open_weights=True,
    revision="a2c8ac47f998531948d4cbe32a0b577a7037a5e3",
    release_date="2022-02-09",
    n_parameters=335726080,
    n_embedding_parameters=32_899_072,
    memory_usage_mb=639,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/gtr-t5-large",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        "MSMARCO",
        "MSMARCOHardNegatives",
        "NanoMSMARCORetrieval",
        "MSMARCO-PL",  # translation not trained on
        "mMARCO-NL",  # translation not trained on
        "NQ",
        "NQ-NL",  # translation not trained on
        "NQHardNegatives",
        "NanoNQRetrieval",
        "NQ-PL",  # translation not trained on
        "Community QA",
    },
    citation=GTR_CITATION,
)

gtr_t5_xl = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/gtr-t5-xl",
    model_type=["dense"],
    languages=["eng-Latn"],  # in format eng-Latn
    open_weights=True,
    revision="23a8d667a1ad2578af181ce762867003c498d1bf",
    release_date="2022-02-09",
    n_parameters=1241696256,
    n_embedding_parameters=32_899_072,
    memory_usage_mb=2367,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/gtr-t5-xl",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        "MSMARCO",
        "MSMARCOHardNegatives",
        "NanoMSMARCORetrieval",
        "MSMARCO-PL",  # translation not trained on
        "mMARCO-NL",  # translation not trained on
        "NQ",
        "NQ-NL",  # translation not trained on
        "NQHardNegatives",
        "NanoNQRetrieval",
        "NQ-PL",  # translation not trained on
        "Community QA",
    },
    citation=GTR_CITATION,
)
gtr_t5_xxl = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/gtr-t5-xxl",
    model_type=["dense"],
    languages=["eng-Latn"],  # in format eng-Latn
    open_weights=True,
    revision="73f2a9156a3dcc2194dfdb2bf201cd7d17e17884",
    release_date="2022-02-09",
    n_parameters=4865577984,
    n_embedding_parameters=32899072,
    memory_usage_mb=9279,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/gtr-t5-xxl",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        "MSMARCO",
        "MSMARCOHardNegatives",
        "NanoMSMARCORetrieval",
        "MSMARCO-PL",  # translation not trained on
        "mMARCO-NL",  # translation not trained on
        "NQ",
        "NQ-NL",  # translation not trained on
        "NQHardNegatives",
        "NanoNQRetrieval",
        "NQ-PL",  # translation not trained on
        "Community QA",
    },
    citation=GTR_CITATION,
)

gtr_t5_base = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/gtr-t5-base",
    model_type=["dense"],
    languages=["eng-Latn"],  # in format eng-Latn
    open_weights=True,
    revision="7027e9594267928589816394bdd295273ddc0739",
    release_date="2022-02-09",
    n_parameters=110218368,
    n_embedding_parameters=24_674_304,
    memory_usage_mb=209,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/sentence-transformers/gtr-t5-base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        "MSMARCO",
        "MSMARCOHardNegatives",
        "NanoMSMARCORetrieval",
        "MSMARCO-PL",  # translation not trained on
        "mMARCO-NL",  # translation not trained on
        "NQ",
        "NQ-NL",  # translation not trained on
        "NQHardNegatives",
        "NanoNQRetrieval",
        "NQ-PL",  # translation not trained on
        "Community QA",
    },
    citation=GTR_CITATION,
)

static_retrieval_mrl_en_v1 = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/static-retrieval-mrl-en-v1",
    revision="f60985c706f192d45d218078e49e5a8b6f15283a",
    release_date="2024-10-24",
    languages=["eng-Latn"],
    n_parameters=3_125_4528,
    n_embedding_parameters=31254528,
    memory_usage_mb=119,
    max_tokens=np.inf,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://huggingface.co/sentence-transformers/static-retrieval-mrl-en-v1/blob/main/train.py",
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/sentence-transformers/static-retrieval-mrl-en-v1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets={
        "MSMARCO",
        # gooaq
        # s2orc
        # allnli
        # paq
        # trivia-qa
        # swim-ir-monolingual
        # PubMedQA
        # swim
        "MIRACLRetrieval",
        "MultiLongDocRetrieval",
        "MrTidyRetrieval",
    },
    modalities=["text"],
    model_type=["dense"],
)

multi_qa_mpnet_base_dot_v1 = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sentence-transformers/multi-qa-mpnet-base-dot-v1",
    revision="3af7c6da5b3e1bea796ef6c97fe237538cbe6e7f",
    release_date="2021-08-23",
    languages=["eng-Latn"],
    n_parameters=109486978,
    n_embedding_parameters=23444736,
    memory_usage_mb=418.0,
    max_tokens=512,
    embed_dim=768,
    license=None,
    open_weights=True,
    public_training_code="https://huggingface.co/sentence-transformers/multi-qa-mpnet-base-dot-v1/blob/main/train_script.py",
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/sentence-transformers/multi-qa-mpnet-base-dot-v1",
    similarity_fn_name=ScoringFunction.DOT_PRODUCT,
    use_instructions=False,
    training_datasets={
        "MSMARCO",
        "YahooAnswersTopicsClassification",
        "NQ",
    },
    adapted_from="microsoft/mpnet-base",
    modalities=["text"],
    model_type=["dense"],
)



from .humests22 import HUMESTS22
from .indic_crosslingual_sts import IndicCrosslingualSTS
from .sem_rel24_sts import SemRel24STS
from .sts17_crosslingual_sts import STS17Crosslingual
from .sts17_multilingual_visual_sts import STS17MultilingualVisualSTS
from .sts22_crosslingual_sts import STS22CrosslingualSTS, STS22CrosslingualSTSv2
from .sts_benchmark_multilingual_sts import STSBenchmarkMultilingualSTS
from .sts_benchmark_multilingual_visual_sts import STSBenchmarkMultilingualVisualSTS

__all__ = [
    "HUMESTS22",
    "IndicCrosslingualSTS",
    "STS17Crosslingual",
    "STS17MultilingualVisualSTS",
    "STS22CrosslingualSTS",
    "STS22CrosslingualSTSv2",
    "STSBenchmarkMultilingualSTS",
    "STSBenchmarkMultilingualVisualSTS",
    "SemRel24STS",
]



from __future__ import annotations

from collections.abc import Iterator, Sequence
from dataclasses import dataclass, field
from functools import lru_cache
from typing import TYPE_CHECKING, Literal, cast

import huggingface_hub
import pandas as pd

from mteb.abstasks.abstask import AbsTask
from mteb.types import StrURL

if TYPE_CHECKING:
    from mteb.abstasks.aggregated_task import AbsTaskAggregate
    from mteb.results import BenchmarkResults


@lru_cache
def _get_benchmarks_on_leaderboard() -> set[str]:
    from mteb.leaderboard.benchmark_selector import (
        GP_BENCHMARK_ENTRIES,
        R_BENCHMARK_ENTRIES,
        MenuEntry,
    )

    entries = GP_BENCHMARK_ENTRIES + R_BENCHMARK_ENTRIES

    def __extract_benchmarks(
        entries: Sequence[Benchmark | MenuEntry],
    ) -> list[Benchmark]:
        benchmarks = []
        for entry in entries:
            if isinstance(entry, Benchmark):
                benchmarks.append(entry)
            else:
                benchmarks.extend(__extract_benchmarks(entry.benchmarks))
        return benchmarks

    names = {benchmark.name for benchmark in __extract_benchmarks(entries)}

    return names


@dataclass
class Benchmark:
    """A benchmark object intended to run a certain benchmark within MTEB.

    Args:
        name: The name of the benchmark
        aliases: Alternative names for the benchmark
        tasks: The tasks within the benchmark.
        description: A description of the benchmark, should include its intended goal and potentially a description of its construction
        reference: A link reference, to a source containing additional information typically to a paper, leaderboard or github.
        citation: A bibtex citation
        contacts: The people to contact in case of a problem in the benchmark, preferably a GitHub handle.

    Examples:
        >>> Benchmark(
        ...     name="MTEB(custom)",
        ...     tasks=mteb.get_tasks(
        ...         tasks=["AmazonCounterfactualClassification", "AmazonPolarityClassification"],
        ...         languages=["eng"],
        ...     ),
        ...     description="A custom benchmark"
        ... )
    """

    name: str
    tasks: Sequence[AbsTask]
    aliases: Sequence[str] = field(default_factory=tuple)
    description: str | None = None
    reference: StrURL | None = None
    citation: str | None = None
    contacts: list[str] | None = None
    icon: str | None = None
    display_name: str | None = None
    language_view: list[str] | Literal["all"] = field(default_factory=list)

    @property
    def display_on_leaderboard(self) -> bool:
        """Whether the benchmark should be displayed on the leaderboard."""
        benchmarks_on_leaderboard = _get_benchmarks_on_leaderboard()
        return self.name in benchmarks_on_leaderboard

    def __iter__(self) -> Iterator[AbsTask]:
        return iter(self.tasks)

    def __len__(self) -> int:
        return len(self.tasks)

    def __getitem__(self, index: int) -> AbsTask:
        return self.tasks[index]

    def _create_summary_table(
        self, benchmark_results: BenchmarkResults
    ) -> pd.DataFrame:
        """Create summary table. Called by the leaderboard app.

        Returns:
            A pandas DataFrame representing the summary results.
        """
        from mteb.benchmarks._create_table import (
            _create_summary_table_from_benchmark_results,
        )

        return _create_summary_table_from_benchmark_results(benchmark_results)

    def _create_per_task_table(
        self, benchmark_results: BenchmarkResults
    ) -> pd.DataFrame:
        """Create per-task table. Called by the leaderboard app.

        Returns:
            A pandas DataFrame representing the per-task results.
        """
        from mteb.benchmarks._create_table import (
            _create_per_task_table_from_benchmark_results,
        )

        return _create_per_task_table_from_benchmark_results(benchmark_results)

    def _create_per_language_table(
        self, benchmark_results: BenchmarkResults
    ) -> pd.DataFrame:
        """Create per-language table. Called by the leaderboard app.

        Returns:
            A pandas DataFrame representing the per-language results.
        """
        from mteb.benchmarks._create_table import (
            _create_per_language_table_from_benchmark_results,
        )

        if self.language_view == "all" or len(self.language_view) > 0:
            return _create_per_language_table_from_benchmark_results(
                benchmark_results, self.language_view
            )
        else:
            no_results_frame = pd.DataFrame(
                {
                    "No results": [
                        "The per-language table is not available for this benchmark."
                    ]
                }
            )
            return no_results_frame

    def push_collection_to_hub(
        self,
        hf_username: str,
        collection_name: str | None = None,
    ) -> None:
        """Push the benchmark collection to Hugging Face Hub.

        Args:
            hf_username: Hugging Face username or organization name
            collection_name: Name for the collection on Hugging Face Hub. If not provided, the benchmark name will be used.
        """
        collections = huggingface_hub.list_collections(owner=hf_username)
        collection_name = collection_name or self.name
        existing_collection = None
        for collection in collections:
            if collection.title == collection_name:
                existing_collection = collection
                break

        if existing_collection is None:
            description = self.description
            if description and len(description) > 150:
                description = description[:147] + "..."
            collection = huggingface_hub.create_collection(
                title=collection_name,
                namespace=hf_username,
                # hf collections have a 150 character limit for description, so we truncate it if it's too long
                description=description if description else None,
            )
        else:
            # list collections would output only 4 items
            collection = huggingface_hub.get_collection(
                collection_slug=existing_collection.slug
            )

        existing_items = {item.item_id for item in collection.items}

        for task in self.tasks:
            tasks = (
                cast("AbsTaskAggregate", task).tasks if task.is_aggregate else [task]
            )
            for benchmark_task in tasks:
                task_path = benchmark_task.metadata.dataset["path"]
                if task_path in existing_items:
                    continue
                huggingface_hub.add_collection_item(
                    collection_slug=collection.slug,
                    item_id=task_path,
                    item_type="dataset",
                )
                existing_items.add(task_path)

    def __repr__(self) -> str:
        n_tasks = len(self.tasks)
        max_len = 50
        desc = self.description if self.description else ""
        desc = f"'{desc[:max_len]}..." if len(desc) > max_len else f"'{desc}'"
        return f"{self.__class__.__name__}(name='{self.name}', description={desc}, tasks=[...] (#{n_tasks}), ...)"


class RtebBenchmark(Benchmark):
    """Wrapper for RTEB benchmark."""

    def _create_summary_table(
        self, benchmark_results: BenchmarkResults
    ) -> pd.DataFrame:
        from mteb.benchmarks._create_table import (
            _create_summary_table_mean_public_private,
        )

        joint_table = _create_summary_table_mean_public_private(
            benchmark_results, exclude_private_from_borda=True
        )
        # issue 3902: temporary remove the private column from RTEB summary table
        if "Mean (Private)" in joint_table.columns:
            joint_table = joint_table.drop(columns=["Mean (Private)"])
        # For RTEB: all tasks are Retrieval type, so Retrieval column = Mean (Task)
        # but due to 3902, if Private column existed, Mean (Task) was the mean of Public and Private so instead we drop Mean (Task) and rename Mean (Public) to Mean (Task)
        joint_table = joint_table.rename(columns={"Retrieval": "Mean (Task)"})
        if "Mean (Task)" in joint_table.columns:
            joint_table = joint_table.drop(columns=["Mean (Task)"])
        joint_table = joint_table.rename(columns={"Mean (Public)": "Mean (Task)"})

        return joint_table


class HUMEBenchmark(Benchmark):
    """Wrapper for HUME benchmark."""

    def _create_summary_table(
        self, benchmark_results: BenchmarkResults
    ) -> pd.DataFrame:
        from mteb.benchmarks._create_table import _create_summary_table_mean_subset

        return _create_summary_table_mean_subset(benchmark_results)


class MIEBBenchmark(Benchmark):
    """Wrapper for MIEB benchmark."""

    def _create_summary_table(
        self, benchmark_results: BenchmarkResults
    ) -> pd.DataFrame:
        from mteb.benchmarks._create_table import _create_summary_table_mean_task_type

        return _create_summary_table_mean_task_type(
            benchmark_results, mean_column_name="Mean (Task)"
        )


class VidoreBenchmark(Benchmark):
    """Wrapper for Vidore3 benchmark."""

    def _create_vidore_summary_table(
        self, benchmark_results: BenchmarkResults
    ) -> pd.DataFrame:
        """Create summary table from BenchmarkResults.

        Returns a DataFrame with one row per model containing summary statistics
        and task type averages. Customized for Vidore benchmark.

        Args:
            benchmark_results: BenchmarkResults object containing model results

        Returns:
            DataFrame with model summaries, ready for styling in the leaderboard
        """
        import mteb
        from mteb.benchmarks._create_table import (
            _format_max_tokens,
            _format_n_active_parameters,
            _format_n_parameters,
            _get_means_per_types,
            _split_on_capital,
        )
        from mteb.get_tasks import get_task

        data = benchmark_results.to_dataframe(format="long")

        if data.empty:
            no_results_frame = pd.DataFrame(
                {"No results": ["You can try relaxing your criteria"]}
            )
            return no_results_frame
        public_task_name = benchmark_results._filter_tasks(is_public=True).task_names
        private_task_name = benchmark_results._filter_tasks(is_public=False).task_names
        # Convert to DataFrame and pivot
        per_task = data.pivot(index="model_name", columns="task_name", values="score")

        # Remove models with no scores
        to_remove = per_task.isna().all(axis="columns")
        if to_remove.all():
            no_results_frame = pd.DataFrame(
                {"No results": ["You can try relaxing your criteria"]}
            )
            return no_results_frame

        models_to_remove = list(per_task[to_remove].index)
        per_task = per_task.drop(models_to_remove, axis=0)

        # Calculate means by task type
        mean_per_type = _get_means_per_types(per_task)
        mean_per_type = mean_per_type.pivot(
            index="model_name", columns="task_type", values="score"
        )
        mean_per_type.columns = [
            _split_on_capital(column) for column in mean_per_type.columns
        ]

        # Calculate overall means
        public_mean = per_task[public_task_name].mean(skipna=False, axis=1)
        private_mean = per_task[private_task_name].mean(skipna=False, axis=1)

        # Build joint table
        joint_table = mean_per_type.copy()
        joint_table.insert(1, "mean(public)", public_mean)
        joint_table.insert(2, "mean(private)", private_mean)
        task_type = get_task(
            per_task.columns[0]
        ).metadata.type  # "DocumentUnderstanding"
        joint_table = joint_table.sort_values(
            [_split_on_capital(task_type), "mean(public)", "mean(private)"],
            ascending=False,
        )

        joint_table = joint_table.reset_index()

        # Add model metadata
        model_metas = joint_table["model_name"].map(mteb.get_model_meta)
        joint_table = joint_table[model_metas.notna()]
        joint_table["model_link"] = model_metas.map(lambda m: m.reference)

        # Insert model metadata columns
        joint_table.insert(
            1,
            "Max Tokens",
            model_metas.map(lambda m: _format_max_tokens(m.max_tokens)),
        )
        joint_table.insert(
            1,
            "Embedding Dimensions",
            model_metas.map(lambda m: int(m.embed_dim) if m.embed_dim else None),
        )
        joint_table.insert(
            1,
            "Total Parameters (B)",
            model_metas.map(lambda m: _format_n_parameters(m.n_parameters)),
        )
        joint_table.insert(
            1,
            "Active Parameters (B)",
            model_metas.map(
                lambda m: _format_n_active_parameters(m.n_active_parameters)
            ),
        )

        # Add release date from model metadata
        joint_table["Release Date"] = model_metas.map(
            lambda m: str(m.release_date) if m.release_date else None
        )

        # Clean up model names (remove HF organization)
        joint_table["model_name"] = joint_table["model_name"].map(
            lambda name: name.split("/")[-1]
        )

        # Add markdown links to model names
        name_w_link = (
            "[" + joint_table["model_name"] + "](" + joint_table["model_link"] + ")"
        )
        joint_table["model_name"] = joint_table["model_name"].mask(
            joint_table["model_link"].notna(), name_w_link
        )
        joint_table = joint_table.drop(columns=["model_link"])

        # Rename columns
        rename_dict = {
            "model_name": "Model",
            "mean(public)": "Mean (Public)",
            "mean(private)": "Mean (Private)",
        }

        joint_table = joint_table.rename(columns=rename_dict)

        # Add Rank column
        joint_table.insert(
            0, "Rank (Mean Task)", [i + 1 for i in range(len(joint_table))]
        )

        return joint_table

    def _create_summary_table(
        self, benchmark_results: BenchmarkResults
    ) -> pd.DataFrame:
        joint_table = self._create_vidore_summary_table(benchmark_results)
        # For ViDoRe (V1, V2, V3): all tasks are Document Understanding type, so Document Understanding column = Mean (Task)
        joint_table = joint_table.rename(
            columns={"Document Understanding": "Mean (Task)"}
        )
        return joint_table



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class GTZANAudioReranking(AbsTaskRetrieval):
    """GTZAN music genre dataset adapted for audio reranking task.

    The GTZAN dataset consists of 1000 audio tracks each 30 seconds long, containing 10 music genres:
    blues, classical, country, disco, hiphop, jazz, metal, pop, reggae, and rock. Each genre is
    represented by 100 tracks. This version is adapted for audio reranking where given a query audio
    from one of the 10 music genres, the task is to rank positive audio samples (same genre) higher
    than negative samples (different genres).

    Each query has 3 positive examples (same music genre) and 10 negative examples
    (different music genres), creating a focused music genre retrieval scenario
    with 13 total candidates per query. The dataset contains 100 queries providing comprehensive
    evaluation across all 10 music genres.
    """

    metadata = TaskMetadata(
        name="GTZANAudioReranking",
        description="GTZAN music genre dataset adapted for audio reranking. Given a query audio from one of 10 music genres, rank 3 relevant audio samples higher than 10 irrelevant ones from different genres. Contains 100 queries across 10 music genres for comprehensive evaluation.",
        reference="https://www.kaggle.com/datasets/andradaolteanu/gtzan-dataset-music-genre-classification",
        dataset={
            "path": "mteb/GTZANAudioReranking",
            "revision": "e6cf8c1771177dbf88550dbdd8c867d676d15119",
        },
        type="AudioReranking",
        category="a2a",
        modalities=["audio"],
        eval_splits=["test"],
        eval_langs=["zxx-Zxxx"],
        main_score="map_at_1000",
        date=("2001-01-01", "2001-12-31"),
        domains=["Music"],
        task_subtypes=["Music Genre Reranking"],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{1021072,
  author = {Tzanetakis, G. and Cook, P.},
  doi = {10.1109/TSA.2002.800560},
  journal = {IEEE Transactions on Speech and Audio Processing},
  keywords = {Humans;Music information retrieval;Instruments;Computer science;Multiple signal classification;Signal analysis;Pattern recognition;Feature extraction;Wavelet analysis;Cultural differences},
  number = {5},
  pages = {293-302},
  title = {Musical genre classification of audio signals},
  volume = {10},
  year = {2002},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class ToxicConversationsVNClassification(AbsTaskClassification):
    num_samples = 16

    metadata = TaskMetadata(
        name="ToxicConversationsVNClassification",
        description="A translated dataset from Collection of comments from the Civil Comments platform together with annotations if the comment is toxic or not. The process of creating the VN-MTEB (Vietnamese Massive Text Embedding Benchmark) from English samples involves a new automated system: - The system uses large language models (LLMs), specifically Coherence's Aya model, for translation. - Applies advanced embedding models to filter the translations. - Use LLM-as-a-judge to scoring the quality of the samples base on multiple criteria.",
        reference="https://www.kaggle.com/competitions/jigsaw-unintended-bias-in-toxicity-classification/overview",
        dataset={
            "path": "GreenNode/toxic-conversations-50k-vn",
            "revision": "2cc697991407cbbe34e7ef7bc9564449a4a99132",
        },
        type="Classification",
        category="t2c",
        eval_splits=["test"],
        eval_langs=["vie-Latn"],
        main_score="accuracy",
        date=("2025-07-29", "2025-07-30"),
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated and LM verified",
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        bibtex_citation=r"""
@misc{pham2025vnmtebvietnamesemassivetext,
  archiveprefix = {arXiv},
  author = {Loc Pham and Tung Luu and Thu Vo and Minh Nguyen and Viet Hoang},
  eprint = {2507.21500},
  primaryclass = {cs.CL},
  title = {VN-MTEB: Vietnamese Massive Text Embedding Benchmark},
  url = {https://arxiv.org/abs/2507.21500},
  year = {2025},
}
""",
        adapted_from=["ToxicConversationsClassification"],
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )



from mteb.models import ModelMeta, sentence_transformers_loader
from mteb.models.model_meta import ScoringFunction
from mteb.models.sentence_transformer_wrapper import CrossEncoderWrapper

multilingual_langs = [
    "eng-Latn",
    "ara-Arab",
    "jpn-Jpan",
    "spa-Latn",
    "fra-Latn",
    "zho-Hans",
    "fin-Latn",
    "ita-Latn",
    "pol-Latn",
    "deu-Latn",
    "nld-Latn",
    "swe-Latn",
    "ron-Latn",
    "ind-Latn",
    "por-Latn",
]

zeroentropy_zembed_1 = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs={
        "trust_remote_code": True,
        "model_kwargs": {"torch_dtype": "bfloat16"},
    },
    name="zeroentropy/zembed-1",
    revision="10378878bba40172305a1a979db64a413ab7da7b",
    release_date="2026-03-02",
    languages=multilingual_langs,
    n_parameters=4022468096,
    n_embedding_parameters=388956160,
    memory_usage_mb=7672,
    max_tokens=32768,
    embed_dim=2560,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["Sentence Transformers", "PyTorch"],
    reference="https://huggingface.co/zeroentropy/zembed-1",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=None,
    adapted_from="Qwen/Qwen3-4B",
    superseded_by=None,
    modalities=["text"],
    model_type=["dense"],
    citation="""@misc{pipitone2025zeloeloinspiredtrainingmethod,
      title={zELO: ELO-inspired Training Method for Rerankers and Embedding Models},
      author={Nicholas Pipitone and Ghita Houir Alami and Advaith Avadhanam and Anton Kaminskyi and Ashley Khoo},
      year={2025},
      eprint={2509.12541},
      archivePrefix={arXiv},
      primaryClass={cs.AI},
      url={https://arxiv.org/abs/2509.12541},
}""",
    contacts=None,
)


zeroentropy_zerank_1 = ModelMeta(
    loader=CrossEncoderWrapper,
    loader_kwargs={"trust_remote_code": True},
    name="zeroentropy/zerank-1",
    revision="9c848454a9a4561496c5130a9c27f22c6b2c5fc3",
    release_date="2025-07-01",
    languages=["eng-Latn"],
    n_parameters=4022470656,
    n_active_parameters_override=None,
    n_embedding_parameters=388956160,
    memory_usage_mb=7672,
    max_tokens=40960,
    embed_dim=2560,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["Sentence Transformers", "PyTorch"],
    reference="https://huggingface.co/zeroentropy/zerank-1",
    similarity_fn_name=None,
    use_instructions=None,
    training_datasets=None,
    adapted_from="Qwen/Qwen3-4B",
    superseded_by=None,
    modalities=["text"],
    model_type=["cross-encoder"],
    citation="""@misc{pipitone2025zeloeloinspiredtrainingmethod,
      title={zELO: ELO-inspired Training Method for Rerankers and Embedding Models},
      author={Nicholas Pipitone and Ghita Houir Alami and Advaith Avadhanam and Anton Kaminskyi and Ashley Khoo},
      year={2025},
      eprint={2509.12541},
      archivePrefix={arXiv},
      primaryClass={cs.AI},
      url={https://arxiv.org/abs/2509.12541},
}""",
    contacts=None,
)

zeroentropy_zerank_1_small = ModelMeta(
    loader=CrossEncoderWrapper,
    loader_kwargs={"trust_remote_code": True},
    name="zeroentropy/zerank-1-small",
    revision="bfaf6a8852da288e60b7375e79943c7f417ad606",
    release_date="2025-07-01",
    languages=["eng-Latn"],
    n_parameters=1720577024,
    n_active_parameters_override=None,
    n_embedding_parameters=311164928,
    memory_usage_mb=3282,
    max_tokens=40960,
    embed_dim=2048,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["Sentence Transformers", "PyTorch"],
    reference="https://huggingface.co/zeroentropy/zerank-1-small",
    similarity_fn_name=None,
    use_instructions=None,
    training_datasets=None,
    adapted_from="Qwen/Qwen3-4B",
    superseded_by=None,
    modalities=["text"],
    model_type=["cross-encoder"],
    citation="""@misc{pipitone2025zeloeloinspiredtrainingmethod,
      title={zELO: ELO-inspired Training Method for Rerankers and Embedding Models},
      author={Nicholas Pipitone and Ghita Houir Alami and Advaith Avadhanam and Anton Kaminskyi and Ashley Khoo},
      year={2025},
      eprint={2509.12541},
      archivePrefix={arXiv},
      primaryClass={cs.AI},
      url={https://arxiv.org/abs/2509.12541},
}""",
    contacts=None,
)

zeroentropy_zerank_2 = ModelMeta(
    loader=CrossEncoderWrapper,
    loader_kwargs={"trust_remote_code": True},
    name="zeroentropy/zerank-2",
    revision="1b0a8f8c70348f5c553f8b534499fe7104ef9a4e",
    release_date="2025-11-19",
    languages=["eng-Latn"],
    n_parameters=4022470656,
    n_active_parameters_override=None,
    n_embedding_parameters=388956160,
    memory_usage_mb=7672,
    max_tokens=40960,
    embed_dim=2560,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["Sentence Transformers", "PyTorch"],
    reference="https://huggingface.co/zeroentropy/zerank-2",
    similarity_fn_name=None,
    use_instructions=None,
    training_datasets=None,
    adapted_from="Qwen/Qwen3-4B",
    superseded_by=None,
    modalities=["text"],
    model_type=["cross-encoder"],
    citation="""@misc{pipitone2025zeloeloinspiredtrainingmethod,
      title={zELO: ELO-inspired Training Method for Rerankers and Embedding Models},
      author={Nicholas Pipitone and Ghita Houir Alami and Advaith Avadhanam and Anton Kaminskyi and Ashley Khoo},
      year={2025},
      eprint={2509.12541},
      archivePrefix={arXiv},
      primaryClass={cs.AI},
      url={https://arxiv.org/abs/2509.12541},
}""",
    contacts=None,
)



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class GermanSTSBenchmarkSTS(AbsTaskSTS):
    min_score = 0
    max_score = 5

    metadata = TaskMetadata(
        name="GermanSTSBenchmark",
        dataset={
            "path": "mteb/GermanSTSBenchmark",
            "revision": "75829b73cccb64bf86a9f098cbc780d37b786091",
        },
        description="Semantic Textual Similarity Benchmark (STSbenchmark) dataset translated into German. "
        + "Translations were originally done by T-Systems on site services GmbH.",
        reference="https://github.com/t-systems-on-site-services-gmbh/german-STSbenchmark",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["validation", "test"],
        eval_langs=["deu-Latn"],
        main_score="cosine_spearman",
        date=("2023-11-09", "2024-01-24"),
        domains=["News", "Web", "Written"],
        task_subtypes=["Textual Entailment"],
        license="cc-by-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@inproceedings{huggingface:dataset:stsb_multi_mt,
  author = {Philip May},
  title = {Machine translated multilingual STS benchmark dataset.},
  url = {https://github.com/PhilipMay/stsb-multi-mt},
  year = {2021},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class UrbanSound8KAudioReranking(AbsTaskRetrieval):
    """UrbanSound8K dataset adapted for audio reranking task.

    The UrbanSound8K dataset contains 8732 labeled sound excerpts (≤4s) of urban sounds from 10 classes:
    air_conditioner, car_horn, children_playing, dog_bark, drilling, engine_idling, gun_shot, jackhammer,
    siren, and street_music. This version is adapted for audio reranking where given a query audio from
    one of the 10 urban sound classes, the task is to rank positive audio samples (same class) higher
    than negative samples (different classes).

    Each query has 4 positive examples (same urban sound class) and 16 negative examples
    (different urban sound classes), creating a focused urban audio retrieval scenario
    with 20 total candidates per query. The dataset contains 200 queries providing comprehensive
    evaluation across all 10 urban sound categories.
    """

    metadata = TaskMetadata(
        name="UrbanSound8KAudioReranking",
        description="UrbanSound8K urban sound dataset adapted for audio reranking. Given a query audio of urban sounds, rank 4 relevant audio samples higher than 16 irrelevant ones from different urban sound classes. Contains 200 queries across 10 urban sound categories for comprehensive evaluation.",
        reference="https://urbansounddataset.weebly.com/urbansound8k.html",
        dataset={
            "path": "mteb/UrbanSound8KAudioReranking",
            "revision": "39896c7e1c66f809915490a1277198fe5ac3639d",
        },
        type="AudioReranking",
        category="a2a",
        modalities=["audio"],
        eval_splits=["test"],
        eval_langs=["zxx-Zxxx"],
        main_score="map_at_1000",
        date=("2014-11-01", "2014-11-03"),
        domains=["Spoken"],
        task_subtypes=["Environment Sound Reranking"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{Salamon:UrbanSound:ACMMM:14,
  author = {Salamon, Justin and Jacoby, Christopher and Bello, Juan Pablo},
  booktitle = {Proceedings of the 22nd ACM international conference on Multimedia},
  organization = {ACM},
  pages = {1041--1044},
  title = {A Dataset and Taxonomy for Urban Sound Research},
  year = {2014},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class Code1Retrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="Code1Retrieval",
        description="Code retrieval dataset with programming questions paired with C/Python/Go/Ruby code snippets for multi-language code retrieval evaluation.",
        reference="https://huggingface.co/datasets/mteb-private/Code1Retrieval-sample",
        dataset={
            "path": "mteb-private/Code1Retrieval",
            "revision": "94d25599a7e0221484f31749448e5ea217484e41",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2024-01-01", "2024-01-01"),
        domains=["Programming", "Written"],
        task_subtypes=["Code retrieval"],
        license="bsd-3-clause",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
        is_public=False,
        contributed_by="Voyage AI",
    )



from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class UCF101Classification(AbsTaskClassification):
    input_column_name: str = "image"
    samples_per_label: int = 16
    n_experiments: int = 5

    metadata = TaskMetadata(
        name="UCF101",
        description="UCF101 is an action recognition data set of realistic action videos collected from YouTube, having 101 action categories. This version of the dataset does not contain images but images saved frame by frame. Train and test splits are generated based on the authors' first version train/test list.",
        reference="https://huggingface.co/datasets/flwrlabs/ucf101",
        dataset={
            "path": "mteb/ucf101",
            "revision": "e0618988874f6ffbf90af69fa6dccecb9be3deb3",
        },
        type="ImageClassification",
        category="i2c",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2012-01-01",
            "2012-12-01",
        ),  # Estimated range for the collection of reviews
        domains=["Scene"],
        task_subtypes=["Activity recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@misc{soomro2012ucf101dataset101human,
  archiveprefix = {arXiv},
  author = {Khurram Soomro and Amir Roshan Zamir and Mubarak Shah},
  eprint = {1212.0402},
  primaryclass = {cs.CV},
  title = {UCF101: A Dataset of 101 Human Actions Classes From Videos in The Wild},
  url = {https://arxiv.org/abs/1212.0402},
  year = {2012},
}
""",
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType


class CLIPModel(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        **kwargs: Any,
    ):
        from transformers import AutoModel, AutoProcessor

        self.model_name = model_name
        self.device = device
        self.model = AutoModel.from_pretrained(model_name, revision=revision).to(
            self.device
        )
        self.processor = AutoProcessor.from_pretrained(model_name, revision=revision)

    def get_text_embeddings(
        self,
        texts: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        all_text_embeddings = []

        with torch.no_grad():
            for batch in tqdm(
                texts, disable=not show_progress_bar, desc="Text Encoding"
            ):
                inputs = self.processor(
                    text=batch["text"],
                    return_tensors="pt",
                    padding=True,
                    truncation=True,
                )
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                text_outputs = self.model.get_text_features(**inputs)
                # Handle both tensor and BaseModelOutputWithPooling returns
                if hasattr(text_outputs, "pooler_output"):
                    text_outputs = text_outputs.pooler_output
                all_text_embeddings.append(text_outputs.cpu())

        all_text_embeddings = torch.cat(all_text_embeddings, dim=0)
        return all_text_embeddings

    @torch.no_grad()
    def get_image_embeddings(
        self,
        images: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        all_image_embeddings = []

        for batch in tqdm(images, disable=not show_progress_bar, desc="Image Encoding"):
            inputs = self.processor(
                images=batch["image"],
                return_tensors="pt",
                padding=True,
            )
            inputs = {k: v.to(self.device) for k, v in inputs.items()}
            image_outputs = self.model.get_image_features(**inputs)
            # Handle both tensor and BaseModelOutputWithPooling returns
            if hasattr(image_outputs, "pooler_output"):
                image_outputs = image_outputs.pooler_output
            all_image_embeddings.append(image_outputs.cpu())

        all_image_embeddings = torch.cat(all_image_embeddings, dim=0)
        return all_image_embeddings

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        image_embeddings = None
        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "image" in inputs.dataset.features:
            image_embeddings = self.get_image_embeddings(inputs, **kwargs)

        if text_embeddings is not None and image_embeddings is not None:
            if len(text_embeddings) != len(image_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + image_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif image_embeddings is not None:
            return image_embeddings
        raise ValueError


CLIP_CITATION = """
@article{radford2021learning,
  title={Learning Transferable Visual Models From Natural Language Supervision},
  author={Radford, Alec and Kim, Jong Wook and Hallacy, Chris and Ramesh, Aditya and Goh, Gabriel and Agarwal, Sandhini and Sastry, Girish and Askell, Amanda and Mishkin, Pamela and Clark, Jack and Krueger, Gretchen and Sutskever, Ilya},
  journal={arXiv preprint arXiv:2103.00020},
  year={2021}
}"""


clip_vit_large_patch14 = ModelMeta(
    loader=CLIPModel,
    name="openai/clip-vit-large-patch14",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="32bd64288804d66eefd0ccbe215aa642df71cc41",
    release_date="2021-02-26",
    modalities=["image", "text"],
    n_parameters=427616513,
    n_embedding_parameters=37945344,
    memory_usage_mb=1631,
    max_tokens=77,
    embed_dim=768,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/openai/clip-vit-large-patch14",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=None,
    citation=CLIP_CITATION,
)

clip_vit_base_patch32 = ModelMeta(
    loader=CLIPModel,
    name="openai/clip-vit-base-patch32",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="3d74acf9a28c67741b2f4f2ea7635f0aaf6f0268",
    release_date="2021-02-26",
    modalities=["image", "text"],
    n_parameters=151277313,
    n_embedding_parameters=25296896,
    memory_usage_mb=576,
    max_tokens=77,
    embed_dim=512,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/openai/clip-vit-base-patch32",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=None,
    citation=CLIP_CITATION,
)

clip_vit_base_patch16 = ModelMeta(
    loader=CLIPModel,
    name="openai/clip-vit-base-patch16",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="57c216476eefef5ab752ec549e440a49ae4ae5f3",
    release_date="2021-02-26",
    modalities=["image", "text"],
    n_parameters=149620737,
    n_embedding_parameters=25296896,
    memory_usage_mb=576,
    max_tokens=77,
    embed_dim=512,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/openai/clip-vit-base-patch16",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=None,
    citation=CLIP_CITATION,
)



import json
import logging
from copy import copy
from pathlib import Path

import pytest
from datasets.exceptions import DatasetNotFoundError

import mteb
from mteb.abstasks.abstask import AbsTask
from mteb.cache import ResultCache
from mteb.models import ModelMeta
from mteb.models.models_protocols import EncoderProtocol
from tests.mock_models import MockSentenceTransformer
from tests.mock_tasks import (
    MockAggregatedTask,
    MockClassificationTask,
    MockMultilingualRetrievalTask,
    MockRetrievalTask,
)

mock_classification = (MockSentenceTransformer(), MockClassificationTask(), 0.5)
mock_retrieval = (
    MockSentenceTransformer(),
    MockRetrievalTask(),
    pytest.approx(0.63093),
)


@pytest.mark.parametrize(
    "model, task, expected_score",
    [mock_classification, mock_retrieval],
    ids=["mock_classification", "mock_retrieval"],
)
def test_evaluate(model: EncoderProtocol, task: AbsTask, expected_score: float):
    results = mteb.evaluate(model, task, cache=None, co2_tracker=False)

    assert len(results) == 1, "should return exactly one result"
    result = results[0]

    assert result.task_name == task.metadata.name, "results should match the task"
    assert set(result.eval_splits) == set(task.eval_splits), "splits should match task."
    assert result.get_score() == expected_score, (
        "main score should match the expected value"
    )


@pytest.mark.parametrize(
    "model, tasks",
    [(MockSentenceTransformer(), [MockClassificationTask(), MockRetrievalTask()])],
    ids=["mock_clf_and_retrieval"],
)
def test_evaluate_w_multiple_tasks(model: EncoderProtocol, tasks: list[AbsTask]):
    results = mteb.evaluate(model, tasks, cache=None, co2_tracker=False)
    assert len(results) == len(tasks), "should return exactly one result per task"


@pytest.mark.parametrize(
    "model, task, expected_score", [mock_classification], ids=["mock_classification"]
)
def test_evaluate_with_cache(
    model: EncoderProtocol, task: AbsTask, expected_score: float, tmp_path: Path
):
    cache = ResultCache(tmp_path)
    results = mteb.evaluate(model, task, cache=cache, co2_tracker=False)

    # Check if the cache is created
    path = cache.get_task_result_path(
        task.metadata.name,
        results.model_name.replace("/", "__"),
        results.model_revision,
    )
    model_meta_path = path.parent / "model_meta.json"
    assert path.exists() and path.is_file(), "cache file should exist"
    assert path.suffix == ".json", "cache file should be a json file"
    assert model_meta_path.exists(), "no model meta path is saved"

    result = results[0]
    assert result.task_name == task.metadata.name, "results should match the task"
    assert set(result.eval_splits) == set(task.eval_splits), "splits should match task."
    assert result.get_score() == expected_score, (
        "main score should match the expected value"
    )

    # test cache re-use
    cached_results = mteb.evaluate(
        model, task, cache=cache, overwrite_strategy="only-cache"
    )
    cached_result = cached_results[0]
    assert cached_result.task_name == task.metadata.name, (
        "results should match the task"
    )
    assert cached_result.get_score() == expected_score, (
        "main score should match the expected value"
    )


@pytest.mark.parametrize(
    "model, task, expected_score,splits",
    [
        (MockSentenceTransformer(), MockClassificationTask(), 0.5, ["train"])
    ],  # default split is "test" so this will run "train" and then ["test", "train"], also means that expected score can be different
    ids=["mock_classification"],
)
def test_evaluate_w_missing_splits(
    model: EncoderProtocol,
    task: AbsTask,
    expected_score: float,
    splits: list[str],
    tmp_path: Path,
):
    cache = ResultCache(tmp_path)

    task._eval_splits = splits
    # ^ eq. to mteb.get_task("name", splits=[...])

    results = mteb.evaluate(model, task, cache=cache, co2_tracker=False)
    result = results[0]

    assert set(result.eval_splits) == set(splits)
    task._eval_splits = list(set(list(task.metadata.eval_splits) + splits))
    results = mteb.evaluate(model, task, cache=cache)
    updated = results[0]

    assert set(updated.eval_splits) != set(result.eval_splits)
    assert set(updated.eval_splits).issuperset(set(task.metadata.eval_splits))

    assert updated.task_name == task.metadata.name, "results should match the task"
    assert updated.get_score() == expected_score, (
        "main score should match the expected value"
    )


@pytest.mark.parametrize(
    "task", [MockClassificationTask()], ids=["mock_classification"]
)
def test_cache_hit(task: AbsTask):
    """Test that evaluating with 'only-cache' raises an error when there are no cache hit."""
    model = mteb.get_model("mteb/baseline-random-encoder")
    with pytest.raises(
        ValueError,
        match="overwrite_strategy is set to 'only-cache' and the results file exists",
    ):
        mteb.evaluate(model, task, overwrite_strategy="only-cache")


@pytest.mark.parametrize(
    "model, task, expected_score",
    [(MockSentenceTransformer(), MockMultilingualRetrievalTask(), 0.63093)],
    ids=["mock_retrieval"],
)
def test_evaluate_w_missing_subset(
    model: EncoderProtocol, task: AbsTask, expected_score: float, tmp_path: Path
):
    cache = ResultCache(tmp_path)

    hf_subsets = list(task.hf_subsets)
    task.hf_subsets = hf_subsets[:-1]
    # ^ eq. to mteb.get_task("name", splits=[...])

    results = mteb.evaluate(model, task, cache=cache, co2_tracker=False)
    result = results[0]

    assert set(result.hf_subsets) == set(hf_subsets[:1])

    task.hf_subsets = hf_subsets
    results = mteb.evaluate(model, task, cache=cache)
    updated = results[0]

    assert set(updated.hf_subsets) != set(result.hf_subsets)
    assert set(updated.hf_subsets) == set(task.metadata.hf_subsets)

    assert updated.task_name == task.metadata.name, "results should match the task"
    assert updated.get_score() == expected_score, (
        "main score should match the expected value"
    )


@pytest.mark.parametrize(
    "model, task, expected_score, splits",
    [(*mock_classification, ["train"])],
    ids=["mock_classification"],
)
def test_evaluate_overwrites(
    model: EncoderProtocol,
    task: AbsTask,
    expected_score: float,
    splits: list[str],
    tmp_path: Path,
):
    cache = ResultCache(tmp_path)

    task._eval_splits = splits
    # ^ eq. to mteb.get_task("name", splits=[...])

    # run part of a task to make an incomplete result
    results = mteb.evaluate(model, task, cache=cache, co2_tracker=False)

    task._eval_splits = task.metadata.eval_splits  # reset splits to default

    with pytest.raises(ValueError):
        results = mteb.evaluate(
            model, task, cache=cache, overwrite_strategy="only-cache"
        )

    with pytest.raises(ValueError):
        results = mteb.evaluate(model, task, cache=cache, overwrite_strategy="never")

    # should just overwrite
    results = mteb.evaluate(model, task, cache=cache, overwrite_strategy="always")
    assert results[0].get_score() == expected_score, (
        "main score should match the expected value"
    )


def test_evaluate_aggregated_task():
    model = mteb.get_model("mteb/baseline-random-encoder")
    task = MockAggregatedTask()
    mteb.evaluate(model, task, cache=None)


def test_run_private_task_warning(caplog):
    """Test that a warning is correctly logged in an attempt run a private dataset is made"""
    task = mteb.get_task("Code1Retrieval")

    def load_data_dataset_not_found(
        num_proc: int | None,
    ):
        raise DatasetNotFoundError

    task.load_data = load_data_dataset_not_found
    model = mteb.get_model("mteb/baseline-random-encoder")

    with caplog.at_level(logging.WARNING):
        result = mteb.evaluate(model, task, cache=None)
        assert len(result.task_results) == 0
        assert "Dataset for private task 'Code1Retrieval' not found" in caplog.text


def test_run_private_task():
    """Tests that private task is run if it is possible to load the data"""
    task = MockRetrievalTask()
    task_metadata = copy(task.metadata)
    task_metadata.is_public = False
    task.metadata = task_metadata
    model = mteb.get_model("mteb/baseline-random-encoder")
    results = mteb.evaluate(model, task, cache=None, public_only=False)
    assert len(results.task_results) == 1


def test_run_task_raise_error():
    """Test that the error is not caught unintentionally"""
    task = MockRetrievalTask()

    def load_error(
        num_proc: int | None,
    ):
        raise RuntimeError("Test error")

    task.load_data = load_error
    model = mteb.get_model("mteb/baseline-random-encoder")
    with pytest.raises(RuntimeError, match="Test error"):
        mteb.evaluate(model, task, cache=None)


def test_run_list_with_error():
    """Test that errors are correctly suppressed, when specified"""
    error_task = MockRetrievalTask()

    def load_error():
        raise RuntimeError("Test error")

    error_task.load_data = load_error
    task = MockRetrievalTask()

    model = mteb.get_model("mteb/baseline-random-encoder")
    results = mteb.evaluate(model, [error_task, task], cache=None, raise_error=False)
    assert len(results.task_results) == 1
    assert len(results.exceptions) == 1


def test_evaluate_unloads_data_when_not_preloaded():
    """Test that evaluate() unloads data when it was not preloaded."""
    model = MockSentenceTransformer()
    task = MockClassificationTask()

    assert task.data_loaded is False
    mteb.evaluate(model, task, cache=None, co2_tracker=False)
    assert task.data_loaded is False, "evaluate() should unload data it loaded"


def test_evaluate_preserves_preloaded_data_across_multiple_calls():
    """Test that preloaded data persists across multiple evaluate() calls."""
    model = MockSentenceTransformer()
    task = MockClassificationTask()

    task.load_data()
    assert task.data_loaded is True

    mteb.evaluate(model, task, cache=None, co2_tracker=False)
    _ = task.dataset["test"]  # Verify dataset wasn't unloaded

    mteb.evaluate(model, task, cache=None, co2_tracker=False)
    _ = task.dataset["test"]  # Verify dataset persists across multiple calls


def test_evaluate_experiment(tmp_path):
    """Test that evaluate() can be used in an experiment context."""
    model = mteb.get_model(
        "mteb/baseline-random-encoder", test_param=123, test_param2="abc"
    )
    task = MockClassificationTask()
    cache = ResultCache(tmp_path)

    mteb.evaluate(model, task, cache=cache)

    expected_path = (
        Path("results")
        / model.mteb_model_meta.model_name_as_path()
        / model.mteb_model_meta.revision
        / "experiments"
        / "test_param_123__test_param2_abc"
        / "MockClassificationTask.json"
    )
    assert (tmp_path / expected_path).exists()


@pytest.mark.parametrize("embed_dim", [None, 10])
def test_evaluate_mrl(tmp_path, embed_dim):
    """Test that evaluate() can be used in an experiment context."""
    model = mteb.get_model(
        "mteb/baseline-random-encoder",
        embed_dim=embed_dim,
    )
    task = MockRetrievalTask()
    cache = ResultCache(tmp_path)
    mteb.evaluate(model, task, cache=cache)

    model_meta_path = (
        tmp_path
        / "results"
        / model.mteb_model_meta.model_name_as_path()
        / model.mteb_model_meta.revision
    )
    if embed_dim is not None:
        model_meta_path = model_meta_path / "experiments" / "embed_dim_10"
    model_meta_path = model_meta_path / "model_meta.json"
    with model_meta_path.open() as f:
        model_meta_json = json.load(f)
    model_meta_json["loader"] = None  # otherwise meta won't be validated
    model_meta = ModelMeta.model_validate(model_meta_json)
    assert isinstance(model_meta.embed_dim, int)


def test_mrl_unsupported_dim():
    """Test that passing unsupported mrl dim raises an error."""
    # try to load model with mrl, but wrong dim
    with pytest.raises(ValueError):
        mteb.get_model(
            "mteb/baseline-random-encoder",
            embed_dim=100,
        )

    # try to load model that don't support mrl
    with pytest.raises(ValueError):
        mteb.get_model(
            "intfloat/multilingual-e5-small",
            embed_dim=100,
        )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class TAUAcousticScenes2022Mobile(AbsTaskClassification):
    metadata = TaskMetadata(
        name="TAUAcousticScenes2022Mobile",
        description="TAU Urban Acoustic Scenes 2022 Mobile, development dataset consists of 1-second audio recordings from 12 European cities in 10 different acoustic scenes using 4 different devices. This is a stratified subsampled version of the evaluation_setup subset of the original dataset.",
        reference="https://zenodo.org/records/6337421",
        dataset={
            "path": "mteb/tau-acoustic-scenes-2022-mobile-mini",
            "revision": "d0da0ed80d22944c7a5690c4b570683d45c4dfaf",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["test"],
        eval_langs=["zxx-Zxxx"],
        main_score="accuracy",
        date=("2022-03-08", "2022-03-08"),
        domains=[
            "AudioScene",
        ],
        task_subtypes=["Environment Sound Classification"],
        license="not specified",
        annotations_creators="expert-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@dataset{heittola_2022_6337421,
  author = {Toni Heittola and Annamaria Mesaros and Tuomas Virtanen},
  publisher = {Zenodo},
  title = {TAU Urban Acoustic Scenes 2022 Mobile, Development Dataset},
  url = {https://doi.org/10.5281/zenodo.6337421},
  year = {2022},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "scene_label"



from .alloprof_retrieval import AlloprofRetrieval
from .bsard_retrieval import BSARDRetrieval, BSARDRetrievalv2
from .f_qu_ad_retrieval import FQuADRetrieval
from .french1_retrieval import French1Retrieval
from .french_legal1_retrieval import FrenchLegal1Retrieval
from .syntec_retrieval import SyntecRetrieval

__all__ = [
    "AlloprofRetrieval",
    "BSARDRetrieval",
    "BSARDRetrievalv2",
    "FQuADRetrieval",
    "French1Retrieval",
    "FrenchLegal1Retrieval",
    "SyntecRetrieval",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_argu_ana_nl_metadata = dict(
    reference="https://huggingface.co/datasets/clips/beir-nl-arguana",
    dataset={
        "path": "clips/beir-nl-arguana",
        "revision": "4cd085d148fe2cac923bb7758d6ef585926170ba",
    },
    type="Retrieval",
    category="t2t",
    modalities=["text"],
    eval_splits=["test"],
    eval_langs=["nld-Latn"],
    main_score="ndcg_at_10",
    date=("2016-03-01", "2016-03-01"),  # best guess: based on publication date
    domains=["Written", "Non-fiction"],
    task_subtypes=[],
    license="cc-by-sa-4.0",
    annotations_creators="derived",
    dialect=[],
    sample_creation="machine-translated and verified",  # manually checked a small subset
    bibtex_citation=r"""
@misc{banar2024beirnlzeroshotinformationretrieval,
  archiveprefix = {arXiv},
  author = {Nikolay Banar and Ehsan Lotfi and Walter Daelemans},
  eprint = {2412.08329},
  primaryclass = {cs.CL},
  title = {BEIR-NL: Zero-shot Information Retrieval Benchmark for the Dutch Language},
  url = {https://arxiv.org/abs/2412.08329},
  year = {2024},
}
""",
)


class ArguAnaNL(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="ArguAna-NL",
        description="ArguAna involves the task of retrieval of the best counterargument to an argument. ArguAna-NL is "
        "a Dutch translation.",
        adapted_from=["ArguAna"],
        **_argu_ana_nl_metadata,
    )


class ArguAnaNLv2(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="ArguAna-NL.v2",
        description="ArguAna involves the task of retrieval of the best counterargument to an argument. ArguAna-NL is "
        "a Dutch translation. This version adds a Dutch prompt to the dataset.",
        prompt={
            "query": "Gegeven een bewering, vind documenten die de bewering weerleggen"
        },
        adapted_from=["ArguAna-NL"],
        **_argu_ana_nl_metadata,
    )



from mteb.abstasks.pair_classification import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata


class SprintDuplicateQuestionsPC(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="SprintDuplicateQuestions",
        description="Duplicate questions from the Sprint community.",
        reference="https://www.aclweb.org/anthology/D18-1131/",
        dataset={
            "path": "mteb/sprintduplicatequestions-pairclassification",
            "revision": "d66bd1f72af766a5cc4b0ca5e00c162f89e8cc46",
        },
        type="PairClassification",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="max_ap",
        date=(
            "2018-10-01",
            "2018-12-30",
        ),  # not found in the paper or data. This is just a rough guess based on the paper's publication date
        domains=["Programming", "Written"],
        task_subtypes=["Duplicate Detection"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        prompt="Retrieve duplicate questions from Sprint forum",
        bibtex_citation=r"""
@inproceedings{shah-etal-2018-adversarial,
  address = {Brussels, Belgium},
  author = {Shah, Darsh  and
Lei, Tao  and
Moschitti, Alessandro  and
Romeo, Salvatore  and
Nakov, Preslav},
  booktitle = {Proceedings of the 2018 Conference on Empirical Methods in Natural Language Processing},
  doi = {10.18653/v1/D18-1131},
  editor = {Riloff, Ellen  and
Chiang, David  and
Hockenmaier, Julia  and
Tsujii, Jun{'}ichi},
  month = oct # {-} # nov,
  pages = {1056--1063},
  publisher = {Association for Computational Linguistics},
  title = {Adversarial Domain Adaptation for Duplicate Question Detection},
  url = {https://aclanthology.org/D18-1131},
  year = {2018},
}
""",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset = self.dataset.rename_column("sent1", "sentence1")
        self.dataset = self.dataset.rename_column("sent2", "sentence2")



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class German1Retrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="German1Retrieval",
        description="German dialogue retrieval dataset with business conversations and workplace communication scenarios.",
        reference="https://huggingface.co/datasets/mteb-private/German1Retrieval-sample",
        dataset={
            "path": "mteb-private/German1Retrieval",
            "revision": "d83dd140fc8fdf96372a6e9193cd4fd2d5cb72bd",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["deu-Latn"],
        main_score="ndcg_at_10",
        date=("2024-01-01", "2024-01-01"),
        domains=["Written", "Non-fiction"],
        task_subtypes=["Question answering"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
        is_public=False,
        contributed_by="Voyage AI",
    )



from .hebrew_sentiment_analysis import (
    HebrewSentimentAnalysis,
    HebrewSentimentAnalysisV2,
    HebrewSentimentAnalysisV3,
)

__all__ = [
    "HebrewSentimentAnalysis",
    "HebrewSentimentAnalysisV2",
    "HebrewSentimentAnalysisV3",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class TRECDL2019(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="TRECDL2019",
        dataset={
            "path": "whybe-choi/trec-dl-2019",
            "revision": "cbb055c2528c544edf84db8ec6aef1885ec8cfda",
        },
        description="TREC Deep Learning Track 2019 passage ranking task. The task involves retrieving relevant passages from the MS MARCO collection given web search queries. Queries have multi-graded relevance judgments.",
        reference="https://microsoft.github.io/msmarco/TREC-Deep-Learning-2019",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2019-01-01", "2019-12-31"),
        domains=[
            "Encyclopaedic",
            "Academic",
            "Blog",
            "News",
            "Medical",
            "Government",
            "Reviews",
            "Non-fiction",
            "Social",
            "Web",
        ],
        task_subtypes=["Question answering"],
        license="msr-la-nc",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{craswell2020overview,
  author = {Craswell, Nick and Mitra, Bhaskar and Yilmaz, Emine and Campos, Daniel and Voorhees, Ellen M},
  booktitle = {Proceedings of the 28th Text REtrieval Conference (TREC 2019)},
  organization = {NIST},
  title = {Overview of the TREC 2019 deep learning track},
  year = {2020},
}

@article{DBLP:journals/corr/NguyenRSGTMD16,
  archiveprefix = {arXiv},
  author = {Tri Nguyen and
Mir Rosenberg and
Xia Song and
Jianfeng Gao and
Saurabh Tiwary and
Rangan Majumder and
Li Deng},
  bibsource = {dblp computer science bibliography, https://dblp.org},
  biburl = {https://dblp.org/rec/journals/corr/NguyenRSGTMD16.bib},
  eprint = {1611.09268},
  journal = {CoRR},
  timestamp = {Mon, 13 Aug 2018 16:49:03 +0200},
  title = {{MS} {MARCO:} {A} Human Generated MAchine Reading COmprehension Dataset},
  url = {http://arxiv.org/abs/1611.09268},
  volume = {abs/1611.09268},
  year = {2016},
}
""",
        prompt={
            "query": "Given a web search query, retrieve relevant passages that answer the query"
        },
    )


class TRECDL2020(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="TRECDL2020",
        dataset={
            "path": "whybe-choi/trec-dl-2020",
            "revision": "83d7d3532f3fa653f132da188bbaa852d959d6f9",
        },
        description="TREC Deep Learning Track 2020 passage ranking task. The task involves retrieving relevant passages from the MS MARCO collection given web search queries. Queries have multi-graded relevance judgments.",
        reference="https://microsoft.github.io/msmarco/TREC-Deep-Learning-2020",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2020-01-01", "2020-12-31"),
        domains=[
            "Encyclopaedic",
            "Academic",
            "Blog",
            "News",
            "Medical",
            "Government",
            "Reviews",
            "Non-fiction",
            "Social",
            "Web",
        ],
        task_subtypes=["Question answering"],
        license="msr-la-nc",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{craswell2021overview,
  author = {Craswell, Nick and Mitra, Bhaskar and Yilmaz, Emine and Campos, Daniel and Voorhees, Ellen M},
  booktitle = {Proceedings of the 29th Text REtrieval Conference (TREC 2020)},
  organization = {NIST},
  title = {Overview of the TREC 2020 deep learning track},
  year = {2021},
}

@article{DBLP:journals/corr/NguyenRSGTMD16,
  archiveprefix = {arXiv},
  author = {Tri Nguyen and
Mir Rosenberg and
Xia Song and
Jianfeng Gao and
Saurabh Tiwary and
Rangan Majumder and
Li Deng},
  bibsource = {dblp computer science bibliography, https://dblp.org},
  biburl = {https://dblp.org/rec/journals/corr/NguyenRSGTMD16.bib},
  eprint = {1611.09268},
  journal = {CoRR},
  timestamp = {Mon, 13 Aug 2018 16:49:03 +0200},
  title = {{MS} {MARCO:} {A} Human Generated MAchine Reading COmprehension Dataset},
  url = {http://arxiv.org/abs/1611.09268},
  volume = {abs/1611.09268},
  year = {2016},
}
""",
        prompt={
            "query": "Given a web search query, retrieve relevant passages that answer the query"
        },
    )



# RTEB Benchmarks - Retrieval Embedding Benchmark


from mteb.benchmarks.benchmark import RtebBenchmark
from mteb.get_tasks import get_tasks

RTEB_CITATION = r"""@article{rteb2025,
  author = {Liu, Frank and Enevoldsen, Kenneth and Solomatin, Roman and Chung, Isaac and Aarsen, Tom and Fődi, Zoltán},
  title = {Introducing RTEB: A New Standard for Retrieval Evaluation},
  year = {2025},
}"""

removal_note = "\n\nNote: We have temporarily removed the 'Private' column to read more about this decision out the [announcement](https://github.com/embeddings-benchmark/mteb/issues/3934)."

RTEB_MAIN = RtebBenchmark(
    name="RTEB(beta)",
    display_name="RTEB Multilingual",
    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-gui-search.svg",
    tasks=get_tasks(
        tasks=[
            "AILACasedocs",
            "AILAStatutes",
            "LegalSummarization",
            "LegalQuAD",
            "FinanceBenchRetrieval",
            "HC3FinanceRetrieval",
            "FinQARetrieval",
            "AppsRetrieval",
            "DS1000Retrieval",
            "HumanEvalRetrieval",
            "MBPPRetrieval",
            "WikiSQLRetrieval",
            "FreshStackRetrieval",
            "ChatDoctorRetrieval",
            "CUREv1",
            "MIRACLRetrievalHardNegatives",
            # Closed datasets
            "Code1Retrieval",
            "JapaneseCode1Retrieval",
            "EnglishFinance1Retrieval",
            "EnglishFinance2Retrieval",
            "EnglishFinance3Retrieval",
            "EnglishFinance4Retrieval",
            "EnglishHealthcare1Retrieval",
            "French1Retrieval",
            "FrenchLegal1Retrieval",
            "German1Retrieval",
            "GermanHealthcare1Retrieval",
            "GermanLegal1Retrieval",
            "JapaneseLegal1Retrieval",
        ],
    ),
    description="RTEB (ReTrieval Embedding Benchmark) is a comprehensive benchmark for evaluating text retrieval models across multiple specialized domains including legal, finance, code, and healthcare. It contains diverse retrieval tasks designed to test models' ability to understand domain-specific terminology and retrieve relevant documents in specialized contexts across multiple languages. The dataset includes both open and closed datasets, providing a robust evaluation framework for real-world applications. To submit results on private tasks, please create [open an issue](https://github.com/embeddings-benchmark/mteb/issues)."
    + removal_note,
    citation=RTEB_CITATION,
    contacts=["fzowl"],
)

RTEB_ENGLISH = RtebBenchmark(
    name="RTEB(eng, beta)",
    display_name="RTEB English",
    icon="https://github.com/lipis/flag-icons/raw/refs/heads/main/flags/4x3/us.svg",
    tasks=get_tasks(
        tasks=[
            "AILACasedocs",
            "AILAStatutes",
            "LegalSummarization",
            "FinanceBenchRetrieval",
            "HC3FinanceRetrieval",
            "FinQARetrieval",
            "AppsRetrieval",
            "DS1000Retrieval",
            "HumanEvalRetrieval",
            "MBPPRetrieval",
            "WikiSQLRetrieval",
            "FreshStackRetrieval",
            "ChatDoctorRetrieval",
            "CUREv1",
            # Closed datasets
            "Code1Retrieval",
            "EnglishFinance1Retrieval",
            "EnglishFinance2Retrieval",
            "EnglishFinance3Retrieval",
            "EnglishFinance4Retrieval",
            "EnglishHealthcare1Retrieval",
        ],
        languages=["eng"],
    ),
    description="RTEB English is a subset of RTEB containing retrieval tasks in English across legal, finance, code, and healthcare domains. Includes diverse tasks covering specialized domains such as healthcare and finance. The benchmark includes both open and closed datasets, providing a robust evaluation framework for real-world applications. To submit results on private tasks, please create [open an issue](https://github.com/embeddings-benchmark/mteb/issues)."
    + removal_note,
    citation=RTEB_CITATION,
    contacts=["fzowl"],
)

RTEB_FRENCH = RtebBenchmark(
    name="RTEB(fra, beta)",
    display_name="RTEB French",
    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/fr.svg",
    tasks=get_tasks(
        tasks=[
            "CUREv1",
            # Closed datasets
            "French1Retrieval",
            "FrenchLegal1Retrieval",
        ],
        languages=["fra"],
    ),
    description="RTEB French is a subset of RTEB containing retrieval tasks in French across legal and general knowledge domains. The benchmark includes both open and closed datasets, providing a robust evaluation framework for real-world applications. To submit results on private tasks, please create [open an issue](https://github.com/embeddings-benchmark/mteb/issues)."
    + removal_note,
    citation=RTEB_CITATION,
    contacts=["fzowl"],
)

RTEB_GERMAN = RtebBenchmark(
    name="RTEB(deu, beta)",
    display_name="RTEB German",
    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/de.svg",
    tasks=get_tasks(
        tasks=[
            "LegalQuAD",
            # Closed datasets
            "German1Retrieval",
            "GermanHealthcare1Retrieval",
            "GermanLegal1Retrieval",
        ],
    ),
    description="RTEB German is a subset of RTEB containing retrieval tasks in German across legal, healthcare, and business domains. The benchmark includes both open and closed datasets, providing a robust evaluation framework for real-world applications. To submit results on private tasks, please create [open an issue](https://github.com/embeddings-benchmark/mteb/issues)."
    + removal_note,
    citation=RTEB_CITATION,
    contacts=["fzowl"],
)

RTEB_JAPANESE = RtebBenchmark(
    name="RTEB(jpn, beta)",
    display_name="RTEB Japanese",
    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/jp.svg",
    tasks=get_tasks(
        tasks=[
            # Closed datasets
            "JapaneseCode1Retrieval",
            "JapaneseLegal1Retrieval",
        ],
    ),
    description="RTEB Japanese is a subset of RTEB  containing retrieval tasks in Japanese across legal and code domains. The benchmark includes both open and closed datasets, providing a robust evaluation framework for real-world applications. To submit results on private tasks, please create [open an issue](https://github.com/embeddings-benchmark/mteb/issues)."
    + removal_note,
    citation=RTEB_CITATION,
    contacts=["fzowl"],
)

RTEB_FINANCE = RtebBenchmark(
    name="RTEB(fin, beta)",
    display_name="RTEB Finance",
    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-gui-price-tag.svg",
    tasks=get_tasks(
        tasks=[
            "FinanceBenchRetrieval",
            "HC3FinanceRetrieval",
            "FinQARetrieval",
            # Closed datasets
            "EnglishFinance1Retrieval",
            "EnglishFinance2Retrieval",
            "EnglishFinance3Retrieval",
            "EnglishFinance4Retrieval",
        ],
    ),
    description="RTEB Finance is a subset of RTEB  containing retrieval tasks specifically focused on financial domain including finance benchmarks, Q&A, financial document retrieval, and corporate governance. The benchmark includes both open and closed datasets, providing a robust evaluation framework for real-world applications. To submit results on private tasks, please create [open an issue](https://github.com/embeddings-benchmark/mteb/issues)."
    + removal_note,
    citation=RTEB_CITATION,
    contacts=["fzowl"],
)

RTEB_LEGAL = RtebBenchmark(
    name="RTEB(Law, beta)",
    display_name="RTEB Legal",
    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-map-library.svg",
    tasks=get_tasks(
        tasks=[
            "AILACasedocs",
            "AILAStatutes",
            "LegalSummarization",
            "LegalQuAD",
            # Closed datasets
            "FrenchLegal1Retrieval",
            "GermanLegal1Retrieval",
            "JapaneseLegal1Retrieval",
        ],
    ),
    description="RTEB Legal is a subset of RTEB containing retrieval tasks specifically focused on legal domain including case documents, statutes, legal summarization, and multilingual legal Q&A. The benchmark includes both open and closed datasets, providing a robust evaluation framework for real-world applications. To submit results on private tasks, please create [open an issue](https://github.com/embeddings-benchmark/mteb/issues)."
    + removal_note,
    citation=RTEB_CITATION,
    contacts=["fzowl"],
)

RTEB_CODE = RtebBenchmark(
    name="RTEB(Code, beta)",
    display_name="RTEB Code",
    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-tech-electronics.svg",
    tasks=get_tasks(
        tasks=[
            "AppsRetrieval",
            "DS1000Retrieval",
            "HumanEvalRetrieval",
            "MBPPRetrieval",
            "WikiSQLRetrieval",
            "FreshStackRetrieval",
            # Closed datasets
            "Code1Retrieval",
            "JapaneseCode1Retrieval",
        ],
    ),
    description="RTEB Code is a subset of RTEB containing retrieval tasks specifically focused on programming and code domains including algorithmic problems, data science tasks, code evaluation, SQL retrieval, and multilingual code retrieval. The benchmark includes both open and closed datasets, providing a robust evaluation framework for real-world applications. To submit results on private tasks, please create [open an issue](https://github.com/embeddings-benchmark/mteb/issues)."
    + removal_note,
    citation=RTEB_CITATION,
    contacts=["fzowl"],
)

RTEB_HEALTHCARE = RtebBenchmark(
    name="RTEB(Health, beta)",
    display_name="RTEB Healthcare",
    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-map-hospital.svg",
    tasks=get_tasks(
        tasks=[
            "ChatDoctorRetrieval",
            "CUREv1",
            # Closed datasets
            "EnglishHealthcare1Retrieval",
            "GermanHealthcare1Retrieval",
        ],
    ),
    description="RTEB Healthcare is a subset of RTEB containing retrieval tasks specifically focused on healthcare and medical domains including medical Q&A, healthcare information retrieval, cross-lingual medical retrieval, and multilingual medical consultation. The benchmark includes both open and closed datasets, providing a robust evaluation framework for real-world applications. To submit results on private tasks, please create [open an issue](https://github.com/embeddings-benchmark/mteb/issues)."
    + removal_note,
    citation=RTEB_CITATION,
    contacts=["fzowl"],
)



from mteb.abstasks.aggregate_task_metadata import AggregateTaskMetadata
from mteb.abstasks.aggregated_task import AbsTaskAggregate
from mteb.tasks.sts.multilingual.sts_benchmark_multilingual_visual_sts import (
    STSBenchmarkMultilingualVisualSTS,
)

task_list_multi = [
    STSBenchmarkMultilingualVisualSTS().filter_languages(
        languages=[
            "deu",
            "spa",
            "fra",
            "ita",
            "nld",
            "pol",
            "por",
            "rus",
            "cmn",
        ],
        hf_subsets=[
            "de",
            "es",
            "fr",
            "it",
            "nl",
            "pl",
            "pt",
            "ru",
            "zh",
        ],
    )
]


class STSBenchmarkMultilingualVisualSTSMultilingual(AbsTaskAggregate):
    metadata = AggregateTaskMetadata(
        name="VisualSTS-b-Multilingual",
        description="STSBenchmarkMultilingualVisualSTS multilingual.",
        reference="https://arxiv.org/abs/2402.08183/",
        tasks=task_list_multi,
        category="i2i",
        license="not specified",
        modalities=["image"],
        annotations_creators="human-annotated",
        dialect=[""],
        sample_creation="rendered",
        main_score="cosine_spearman",
        type="VisualSTS(multi)",
        eval_splits=["test"],
        eval_langs=[
            "deu-Latn",
            "spa-Latn",
            "fra-Latn",
            "ita-Latn",
            "nld-Latn",
            "pol-Latn",
            "por-Latn",
            "rus-Cyrl",
            "cmn-Hans",
        ],
        bibtex_citation=r"""
@article{xiao2024pixel,
  author = {Xiao, Chenghao and Huang, Zhuoxu and Chen, Danlu and Hudson, G Thomas and Li, Yizhi and Duan, Haoran and Lin, Chenghua and Fu, Jie and Han, Jungong and Moubayed, Noura Al},
  journal = {arXiv preprint arXiv:2402.08183},
  title = {Pixel Sentence Representation Learning},
  year = {2024},
}
""",
    )



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class SickFrSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="SICKFr",
        dataset={
            "path": "Lajavaness/SICK-fr",
            "revision": "e077ab4cf4774a1e36d86d593b150422fafd8e8a",
        },
        description="SICK dataset french version",
        reference="https://huggingface.co/datasets/Lajavaness/SICK-fr",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["validation", "test"],
        eval_langs=["fra-Latn"],
        main_score="cosine_spearman",
        date=("2014-01-01", "2024-12-31"),
        domains=["Web", "Written"],
        task_subtypes=["Textual Entailment"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@inproceedings{marelli-etal-2014-sick,
  address = {Reykjavik, Iceland},
  author = {Marelli, Marco  and
Menini, Stefano  and
Baroni, Marco  and
Bentivogli, Luisa  and
Bernardi, Raffaella  and
Zamparelli, Roberto},
  booktitle = {Proceedings of the Ninth International Conference on Language Resources and Evaluation ({LREC}'14)},
  month = may,
  pages = {216--223},
  publisher = {European Language Resources Association (ELRA)},
  title = {A {SICK} cure for the evaluation of compositional distributional semantic models},
  url = {http://www.lrec-conf.org/proceedings/lrec2014/pdf/363_Paper.pdf},
  year = {2014},
}
""",
    )

    min_score = 0
    max_score = 5

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset = self.dataset.rename_columns(
            {
                "sentence_A": "sentence1",
                "sentence_B": "sentence2",
                "relatedness_score": "score",
                "Unnamed: 0": "id",
            }
        )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class TurkishMovieSentimentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="TurkishMovieSentimentClassification",
        description="Turkish Movie Review Dataset",
        reference="https://www.win.tue.nl/~mpechen/publications/pubs/MT_WISDOM2013.pdf",
        dataset={
            "path": "asparius/Turkish-Movie-Review",
            "revision": "409a4415cce5f6bcfca6d5f3ca3c408211ca00b3",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["tur-Latn"],
        main_score="accuracy",
        date=("2013-01-01", "2013-08-11"),
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{Demirtas2013CrosslingualPD,
  author = {Erkin Demirtas and Mykola Pechenizkiy},
  booktitle = {wisdom},
  title = {Cross-lingual polarity detection with machine translation},
  url = {https://api.semanticscholar.org/CorpusID:3912960},
  year = {2013},
}
""",
        superseded_by="TurkishMovieSentimentClassification.v2",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )


class TurkishMovieSentimentClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="TurkishMovieSentimentClassification.v2",
        description="Turkish Movie Review Dataset This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://www.win.tue.nl/~mpechen/publications/pubs/MT_WISDOM2013.pdf",
        dataset={
            "path": "mteb/turkish_movie_sentiment",
            "revision": "8ef5ce93ff2504de7fc46776317b78bdd8db47f2",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["tur-Latn"],
        main_score="accuracy",
        date=("2013-01-01", "2013-08-11"),
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{Demirtas2013CrosslingualPD,
  author = {Erkin Demirtas and Mykola Pechenizkiy},
  booktitle = {wisdom},
  title = {Cross-lingual polarity detection with machine translation},
  url = {https://api.semanticscholar.org/CorpusID:3912960},
  year = {2013},
}
""",
        adapted_from=["TurkishMovieSentimentClassification"],
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "en": ["eng-Latn"],
    "fi": ["fin-Latn"],
    "pt": ["por-Latn"],
}


class EuroPIRQRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="EuroPIRQRetrieval",
        description="The EuroPIRQ retrieval dataset is a multilingual collection designed for evaluating retrieval and cross-lingual retrieval tasks. Dataset contains 10,000 parallel passages & 100 parallel queries (synthetic) in three languages: English, Portuguese, and Finnish, constructed from the European Union's DGT-Acquis corpus.",
        reference="https://huggingface.co/datasets/eherra/EuroPIRQ-retrieval",
        dataset={
            "path": "eherra/EuroPIRQ-retrieval",
            "revision": "59225ed25fbcea2185e1acbc8c3c80f1a8cd8341",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="ndcg_at_10",
        date=("2025-12-01", "2025-12-31"),
        domains=["Legal"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="LM-generated and reviewed",
        dialect=[],
        sample_creation="found",
        is_public=True,
        bibtex_citation=r"""
@misc{eherra_2025_europirq,
  author = { {Elias Herranen} },
  publisher = { Hugging Face },
  title = { EuroPIRQ: European Parallel Information Retrieval Queries },
  url = { https://huggingface.co/datasets/eherra/EuroPIRQ-retrieval },
  year = {2025},
}
""",
    )



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb._requires_package import (
    requires_image_dependencies,
    requires_package,
)
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_implementations.colpali_models import COLPALI_TRAINING_DATA
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType


NOMIC_LANGUAGES = [
    "deu-Latn",  # German
    "spa-Latn",  # Spanish
    "eng-Latn",  # English
    "fra-Latn",  # French
    "ita-Latn",  # Italian
]

CITATION = """
@misc{nomicembedmultimodal2025,
  title={Nomic Embed Multimodal: Interleaved Text, Image, and Screenshots for Visual Document Retrieval},
  author={Nomic Team},
  year={2025},
  publisher={Nomic AI},
  url={https://www.nomic.ai/news/nomic-embed-multimodal}
}"""

# https://huggingface.co/datasets/nomic-ai/colpali-queries-mined-20250321-by-source
TRAINING_DATA = COLPALI_TRAINING_DATA | {"VDRMultilingualRetrieval"}


logger = logging.getLogger(__name__)


class BiQwen2_5Wrapper(AbsEncoder):  # noqa: N801
    """Wrapper for BiQwen2_5 dense (single-vector) embedding model."""

    def __init__(
        self,
        model_name: str,
        revision: str | None = None,
        device: str | None = None,
        base_revision: str | None = None,
        **kwargs,
    ):
        requires_image_dependencies()
        requires_package(
            self, "colpali_engine", model_name, "pip install mteb[colpali_engine]"
        )
        from colpali_engine.models import BiQwen2_5, BiQwen2_5_Processor

        self.device = device or ("cuda" if torch.cuda.is_available() else "cpu")

        self.mdl = BiQwen2_5.from_pretrained(
            model_name,
            device_map=self.device,
            adapter_kwargs={"revision": revision},
            revision=base_revision,
            attn_implementation="flash_attention_2",  # With this enabled it goes from 0.57382 to 0.58021 on Vidore2ESGReportsHLRetrieval
            **kwargs,
        )
        self.mdl.eval()

        self.processor = BiQwen2_5_Processor.from_pretrained(model_name)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        image_embeddings = None
        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "image" in inputs.dataset.features:
            image_embeddings = self.get_image_embeddings(inputs, **kwargs)

        if text_embeddings is not None and image_embeddings is not None:
            if len(text_embeddings) != len(image_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + image_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif image_embeddings is not None:
            return image_embeddings
        raise ValueError

    def get_image_embeddings(
        self,
        images,
        batch_size: int = 32,
        **kwargs,
    ):
        all_embeds = []

        with torch.no_grad():
            for batch in tqdm(images, desc="Encoding images"):
                inputs = self.processor.process_images(batch["image"])
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                outs = self.mdl(**inputs)
                all_embeds.extend(outs.cpu().to(torch.float32))

        padded = torch.nn.utils.rnn.pad_sequence(
            all_embeds, batch_first=True, padding_value=0
        )
        return padded

    def get_text_embeddings(
        self,
        texts,
        batch_size: int = 32,
        **kwargs,
    ):
        all_embeds = []
        with torch.no_grad():
            for batch in tqdm(texts, desc="Encoding texts"):
                inputs = self.processor.process_queries(batch["text"])
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                outs = self.mdl(**inputs)
                all_embeds.extend(outs.cpu().to(torch.float32))

        padded = torch.nn.utils.rnn.pad_sequence(
            all_embeds, batch_first=True, padding_value=0
        )
        return padded

    def similarity(
        self, a, b
    ):  # Using the processing it goes from 0.57382 to 0.57297 on Vidore2ESGReportsHLRetrieval (without flash attention 2)
        return self.processor.score(a, b, device=self.device)


nomic_embed_multimodal_3b = ModelMeta(
    loader=BiQwen2_5Wrapper,
    loader_kwargs=dict(
        torch_dtype=torch.bfloat16,
        base_revision="66285546d2b821cf421d4f5eb2576359d3770cd3",
    ),
    name="nomic-ai/nomic-embed-multimodal-3b",
    model_type=["dense"],
    languages=NOMIC_LANGUAGES,
    revision="298930bb768c50b91d2799d6f3b0daf46ea52e70",
    release_date="2025-04-15",
    modalities=["image", "text"],
    n_parameters=3_814_490_112,
    n_embedding_parameters=311_164_928,
    memory_usage_mb=6200,
    max_tokens=128000,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/nomic-ai/colpali",
    public_training_data="https://huggingface.co/datasets/nomic-ai/colpali-queries-mined-20250321-by-source",
    framework=["ColPali", "safetensors"],
    reference="https://huggingface.co/nomic-ai/nomic-embed-multimodal-3b",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=TRAINING_DATA,
    citation=CITATION,
)

nomic_embed_multimodal_7b = ModelMeta(
    loader=BiQwen2_5Wrapper,
    loader_kwargs=dict(
        torch_dtype=torch.bfloat16,
        base_revision="cc594898137f460bfe9f0759e9844b3ce807cfb5",
    ),
    name="nomic-ai/nomic-embed-multimodal-7b",
    model_type=["dense"],
    languages=NOMIC_LANGUAGES,
    revision="1291f1b6ca07061b0329df9d5713c09b294be576",
    release_date="2025-04-15",
    modalities=["image", "text"],
    n_parameters=7_827_909_632,
    n_embedding_parameters=544997376,
    memory_usage_mb=14400,
    max_tokens=128000,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/nomic-ai/colpali",
    public_training_data="https://huggingface.co/datasets/nomic-ai/colpali-queries-mined-20250321-by-source",
    framework=["ColPali", "safetensors"],
    reference="https://huggingface.co/nomic-ai/nomic-embed-multimodal-7b",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=TRAINING_DATA,
    citation=CITATION,
)



from mteb.abstasks.clustering_legacy import AbsTaskClusteringLegacy
from mteb.abstasks.task_metadata import TaskMetadata


class HUMEWikiCitiesClustering(AbsTaskClusteringLegacy):
    metadata = TaskMetadata(
        name="HUMEWikiCitiesClustering",
        description="Human evaluation subset of Clustering of Wikipedia articles of cities by country from https://huggingface.co/datasets/wikipedia. Test set includes 126 countries, and a total of 3531 cities.",
        reference="https://huggingface.co/datasets/wikipedia",
        dataset={
            "path": "mteb/mteb-human-wikicities-clustering",
            "revision": "5c46af681d2dfa6d3ee373b7ccb4f153e1b72792",
        },
        type="Clustering",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="v_measure",
        date=("2000-01-01", "2021-12-31"),  # very rough estimate
        domains=["Encyclopaedic", "Written"],
        task_subtypes=[],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@online{wikidump2024,
  author = {Wikimedia Foundation},
  title = {Wikimedia Downloads},
  url = {https://dumps.wikimedia.org},
}
""",
        adapted_from=["WikiCitiesClustering"],
    )
